# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ProcessorResponseCode(object):

    """Implementation of the 'Processor Response Code' enum.

    Processor response code for the non-PayPal payment processor errors.

    Attributes:
        ENUM_0000: TODO: type description here.
        ENUM_00N7: TODO: type description here.
        ENUM_0100: TODO: type description here.
        ENUM_0390: TODO: type description here.
        ENUM_0500: TODO: type description here.
        ENUM_0580: TODO: type description here.
        ENUM_0800: TODO: type description here.
        ENUM_0880: TODO: type description here.
        ENUM_0890: TODO: type description here.
        ENUM_0960: TODO: type description here.
        ENUM_0R00: TODO: type description here.
        ENUM_1000: TODO: type description here.
        ENUM_10BR: TODO: type description here.
        ENUM_1300: TODO: type description here.
        ENUM_1310: TODO: type description here.
        ENUM_1312: TODO: type description here.
        ENUM_1317: TODO: type description here.
        ENUM_1320: TODO: type description here.
        ENUM_1330: TODO: type description here.
        ENUM_1335: TODO: type description here.
        ENUM_1340: TODO: type description here.
        ENUM_1350: TODO: type description here.
        ENUM_1352: TODO: type description here.
        ENUM_1360: TODO: type description here.
        ENUM_1370: TODO: type description here.
        ENUM_1380: TODO: type description here.
        ENUM_1382: TODO: type description here.
        ENUM_1384: TODO: type description here.
        ENUM_1390: TODO: type description here.
        ENUM_1393: TODO: type description here.
        ENUM_5100: TODO: type description here.
        ENUM_5110: TODO: type description here.
        ENUM_5120: TODO: type description here.
        ENUM_5130: TODO: type description here.
        ENUM_5135: TODO: type description here.
        ENUM_5140: TODO: type description here.
        ENUM_5150: TODO: type description here.
        ENUM_5160: TODO: type description here.
        ENUM_5170: TODO: type description here.
        ENUM_5180: TODO: type description here.
        ENUM_5190: TODO: type description here.
        ENUM_5200: TODO: type description here.
        ENUM_5210: TODO: type description here.
        ENUM_5400: TODO: type description here.
        ENUM_5500: TODO: type description here.
        ENUM_5650: TODO: type description here.
        ENUM_5700: TODO: type description here.
        ENUM_5710: TODO: type description here.
        ENUM_5800: TODO: type description here.
        ENUM_5900: TODO: type description here.
        ENUM_5910: TODO: type description here.
        ENUM_5920: TODO: type description here.
        ENUM_5930: TODO: type description here.
        ENUM_5950: TODO: type description here.
        ENUM_6300: TODO: type description here.
        ENUM_7600: TODO: type description here.
        ENUM_7700: TODO: type description here.
        ENUM_7710: TODO: type description here.
        ENUM_7800: TODO: type description here.
        ENUM_7900: TODO: type description here.
        ENUM_8000: TODO: type description here.
        ENUM_8010: TODO: type description here.
        ENUM_8020: TODO: type description here.
        ENUM_8030: TODO: type description here.
        ENUM_8100: TODO: type description here.
        ENUM_8110: TODO: type description here.
        ENUM_8220: TODO: type description here.
        ENUM_9100: TODO: type description here.
        ENUM_9500: TODO: type description here.
        ENUM_9510: TODO: type description here.
        ENUM_9520: TODO: type description here.
        ENUM_9530: TODO: type description here.
        ENUM_9540: TODO: type description here.
        ENUM_9600: TODO: type description here.
        PCNR: TODO: type description here.
        PCVV: TODO: type description here.
        PP06: TODO: type description here.
        PPRN: TODO: type description here.
        PPAD: TODO: type description here.
        PPAB: TODO: type description here.
        PPAE: TODO: type description here.
        PPAG: TODO: type description here.
        PPAI: TODO: type description here.
        PPAR: TODO: type description here.
        PPAU: TODO: type description here.
        PPAV: TODO: type description here.
        PPAX: TODO: type description here.
        PPBG: TODO: type description here.
        PPC2: TODO: type description here.
        PPCE: TODO: type description here.
        PPCO: TODO: type description here.
        PPCR: TODO: type description here.
        PPCT: TODO: type description here.
        PPCU: TODO: type description here.
        PPD3: TODO: type description here.
        PPDC: TODO: type description here.
        PPDI: TODO: type description here.
        PPDV: TODO: type description here.
        PPDT: TODO: type description here.
        PPEF: TODO: type description here.
        PPEL: TODO: type description here.
        PPER: TODO: type description here.
        PPEX: TODO: type description here.
        PPFE: TODO: type description here.
        PPFI: TODO: type description here.
        PPFR: TODO: type description here.
        PPFV: TODO: type description here.
        PPGR: TODO: type description here.
        PPH1: TODO: type description here.
        PPIF: TODO: type description here.
        PPII: TODO: type description here.
        PPIM: TODO: type description here.
        PPIT: TODO: type description here.
        PPLR: TODO: type description here.
        PPLS: TODO: type description here.
        PPMB: TODO: type description here.
        PPMC: TODO: type description here.
        PPMD: TODO: type description here.
        PPNC: TODO: type description here.
        PPNL: TODO: type description here.
        PPNM: TODO: type description here.
        PPNT: TODO: type description here.
        PPPH: TODO: type description here.
        PPPI: TODO: type description here.
        PPPM: TODO: type description here.
        PPQC: TODO: type description here.
        PPRE: TODO: type description here.
        PPRF: TODO: type description here.
        PPRR: TODO: type description here.
        PPS0: TODO: type description here.
        PPS1: TODO: type description here.
        PPS2: TODO: type description here.
        PPS3: TODO: type description here.
        PPS4: TODO: type description here.
        PPS5: TODO: type description here.
        PPS6: TODO: type description here.
        PPSC: TODO: type description here.
        PPSD: TODO: type description here.
        PPSE: TODO: type description here.
        PPTE: TODO: type description here.
        PPTF: TODO: type description here.
        PPTI: TODO: type description here.
        PPTR: TODO: type description here.
        PPTT: TODO: type description here.
        PPTV: TODO: type description here.
        PPUA: TODO: type description here.
        PPUC: TODO: type description here.
        PPUE: TODO: type description here.
        PPUI: TODO: type description here.
        PPUP: TODO: type description here.
        PPUR: TODO: type description here.
        PPVC: TODO: type description here.
        PPVE: TODO: type description here.
        PPVT: TODO: type description here.

    """
    ENUM_0000 = '0000'

    ENUM_00N7 = '00N7'

    ENUM_0100 = '0100'

    ENUM_0390 = '0390'

    ENUM_0500 = '0500'

    ENUM_0580 = '0580'

    ENUM_0800 = '0800'

    ENUM_0880 = '0880'

    ENUM_0890 = '0890'

    ENUM_0960 = '0960'

    ENUM_0R00 = '0R00'

    ENUM_1000 = '1000'

    ENUM_10BR = '10BR'

    ENUM_1300 = '1300'

    ENUM_1310 = '1310'

    ENUM_1312 = '1312'

    ENUM_1317 = '1317'

    ENUM_1320 = '1320'

    ENUM_1330 = '1330'

    ENUM_1335 = '1335'

    ENUM_1340 = '1340'

    ENUM_1350 = '1350'

    ENUM_1352 = '1352'

    ENUM_1360 = '1360'

    ENUM_1370 = '1370'

    ENUM_1380 = '1380'

    ENUM_1382 = '1382'

    ENUM_1384 = '1384'

    ENUM_1390 = '1390'

    ENUM_1393 = '1393'

    ENUM_5100 = '5100'

    ENUM_5110 = '5110'

    ENUM_5120 = '5120'

    ENUM_5130 = '5130'

    ENUM_5135 = '5135'

    ENUM_5140 = '5140'

    ENUM_5150 = '5150'

    ENUM_5160 = '5160'

    ENUM_5170 = '5170'

    ENUM_5180 = '5180'

    ENUM_5190 = '5190'

    ENUM_5200 = '5200'

    ENUM_5210 = '5210'

    ENUM_5400 = '5400'

    ENUM_5500 = '5500'

    ENUM_5650 = '5650'

    ENUM_5700 = '5700'

    ENUM_5710 = '5710'

    ENUM_5800 = '5800'

    ENUM_5900 = '5900'

    ENUM_5910 = '5910'

    ENUM_5920 = '5920'

    ENUM_5930 = '5930'

    ENUM_5950 = '5950'

    ENUM_6300 = '6300'

    ENUM_7600 = '7600'

    ENUM_7700 = '7700'

    ENUM_7710 = '7710'

    ENUM_7800 = '7800'

    ENUM_7900 = '7900'

    ENUM_8000 = '8000'

    ENUM_8010 = '8010'

    ENUM_8020 = '8020'

    ENUM_8030 = '8030'

    ENUM_8100 = '8100'

    ENUM_8110 = '8110'

    ENUM_8220 = '8220'

    ENUM_9100 = '9100'

    ENUM_9500 = '9500'

    ENUM_9510 = '9510'

    ENUM_9520 = '9520'

    ENUM_9530 = '9530'

    ENUM_9540 = '9540'

    ENUM_9600 = '9600'

    PCNR = 'PCNR'

    PCVV = 'PCVV'

    PP06 = 'PP06'

    PPRN = 'PPRN'

    PPAD = 'PPAD'

    PPAB = 'PPAB'

    PPAE = 'PPAE'

    PPAG = 'PPAG'

    PPAI = 'PPAI'

    PPAR = 'PPAR'

    PPAU = 'PPAU'

    PPAV = 'PPAV'

    PPAX = 'PPAX'

    PPBG = 'PPBG'

    PPC2 = 'PPC2'

    PPCE = 'PPCE'

    PPCO = 'PPCO'

    PPCR = 'PPCR'

    PPCT = 'PPCT'

    PPCU = 'PPCU'

    PPD3 = 'PPD3'

    PPDC = 'PPDC'

    PPDI = 'PPDI'

    PPDV = 'PPDV'

    PPDT = 'PPDT'

    PPEF = 'PPEF'

    PPEL = 'PPEL'

    PPER = 'PPER'

    PPEX = 'PPEX'

    PPFE = 'PPFE'

    PPFI = 'PPFI'

    PPFR = 'PPFR'

    PPFV = 'PPFV'

    PPGR = 'PPGR'

    PPH1 = 'PPH1'

    PPIF = 'PPIF'

    PPII = 'PPII'

    PPIM = 'PPIM'

    PPIT = 'PPIT'

    PPLR = 'PPLR'

    PPLS = 'PPLS'

    PPMB = 'PPMB'

    PPMC = 'PPMC'

    PPMD = 'PPMD'

    PPNC = 'PPNC'

    PPNL = 'PPNL'

    PPNM = 'PPNM'

    PPNT = 'PPNT'

    PPPH = 'PPPH'

    PPPI = 'PPPI'

    PPPM = 'PPPM'

    PPQC = 'PPQC'

    PPRE = 'PPRE'

    PPRF = 'PPRF'

    PPRR = 'PPRR'

    PPS0 = 'PPS0'

    PPS1 = 'PPS1'

    PPS2 = 'PPS2'

    PPS3 = 'PPS3'

    PPS4 = 'PPS4'

    PPS5 = 'PPS5'

    PPS6 = 'PPS6'

    PPSC = 'PPSC'

    PPSD = 'PPSD'

    PPSE = 'PPSE'

    PPTE = 'PPTE'

    PPTF = 'PPTF'

    PPTI = 'PPTI'

    PPTR = 'PPTR'

    PPTT = 'PPTT'

    PPTV = 'PPTV'

    PPUA = 'PPUA'

    PPUC = 'PPUC'

    PPUE = 'PPUE'

    PPUI = 'PPUI'

    PPUP = 'PPUP'

    PPUR = 'PPUR'

    PPVC = 'PPVC'

    PPVE = 'PPVE'

    PPVT = 'PPVT'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class StoredPaymentSourcePaymentType(object):

    """Implementation of the 'Stored Payment Source Payment Type' enum.

    Indicates the type of the stored payment_source payment.

    Attributes:
        ONE_TIME: TODO: type description here.
        RECURRING: TODO: type description here.
        UNSCHEDULED: TODO: type description here.

    """
    ONE_TIME = 'ONE_TIME'

    RECURRING = 'RECURRING'

    UNSCHEDULED = 'UNSCHEDULED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PayeePaymentMethodPreference(object):

    """Implementation of the 'Payee Payment Method Preference' enum.

    The merchant-preferred payment methods.

    Attributes:
        UNRESTRICTED: TODO: type description here.
        IMMEDIATE_PAYMENT_REQUIRED: TODO: type description here.

    """
    UNRESTRICTED = 'UNRESTRICTED'

    IMMEDIATE_PAYMENT_REQUIRED = 'IMMEDIATE_PAYMENT_REQUIRED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.vault_venmo_experience_context import VaultVenmoExperienceContext
from paypalserversdk.models.vaulted_digital_wallet_shipping_details import VaultedDigitalWalletShippingDetails


class VaultVenmoRequest(object):

    """Implementation of the 'Vault Venmo Request' model.

    TODO: type model description here.

    Attributes:
        description (str): The description displayed to the consumer on the
            approval flow for a digital wallet, as well as on the merchant
            view of the payment token management experience. exp: PayPal.com.
        shipping (VaultedDigitalWalletShippingDetails): The shipping details.
        permit_multiple_payment_tokens (bool): Create multiple payment tokens
            for the same payer, merchant/platform combination. Use this when
            the customer has not logged in at merchant/platform. The payment
            token thus generated, can then also be used to create the customer
            account at merchant/platform. Use this also when multiple payment
            tokens are required for the same payer, different customer at
            merchant/platform. This helps to identify customers distinctly
            even though they may share the same PayPal account. This only
            applies to PayPal payment source.
        usage_type (str): The usage type associated with a digital wallet
            payment token.
        customer_type (str): The customer type associated with a digital
            wallet payment token. This is to indicate whether the customer
            acting on the merchant / platform is either a business or a
            consumer.
        experience_context (VaultVenmoExperienceContext): Customizes the Vault
            creation flow experience for your customers.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "description": 'description',
        "shipping": 'shipping',
        "permit_multiple_payment_tokens": 'permit_multiple_payment_tokens',
        "usage_type": 'usage_type',
        "customer_type": 'customer_type',
        "experience_context": 'experience_context'
    }

    _optionals = [
        'description',
        'shipping',
        'permit_multiple_payment_tokens',
        'usage_type',
        'customer_type',
        'experience_context',
    ]

    def __init__(self,
                 description=APIHelper.SKIP,
                 shipping=APIHelper.SKIP,
                 permit_multiple_payment_tokens=False,
                 usage_type=APIHelper.SKIP,
                 customer_type=APIHelper.SKIP,
                 experience_context=APIHelper.SKIP):
        """Constructor for the VaultVenmoRequest class"""

        # Initialize members of the class
        if description is not APIHelper.SKIP:
            self.description = description 
        if shipping is not APIHelper.SKIP:
            self.shipping = shipping 
        self.permit_multiple_payment_tokens = permit_multiple_payment_tokens 
        if usage_type is not APIHelper.SKIP:
            self.usage_type = usage_type 
        if customer_type is not APIHelper.SKIP:
            self.customer_type = customer_type 
        if experience_context is not APIHelper.SKIP:
            self.experience_context = experience_context 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        description = dictionary.get("description") if dictionary.get("description") else APIHelper.SKIP
        shipping = VaultedDigitalWalletShippingDetails.from_dictionary(dictionary.get('shipping')) if 'shipping' in dictionary.keys() else APIHelper.SKIP
        permit_multiple_payment_tokens = dictionary.get("permit_multiple_payment_tokens") if dictionary.get("permit_multiple_payment_tokens") else False
        usage_type = dictionary.get("usage_type") if dictionary.get("usage_type") else APIHelper.SKIP
        customer_type = dictionary.get("customer_type") if dictionary.get("customer_type") else APIHelper.SKIP
        experience_context = VaultVenmoExperienceContext.from_dictionary(dictionary.get('experience_context')) if 'experience_context' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(description,
                   shipping,
                   permit_multiple_payment_tokens,
                   usage_type,
                   customer_type,
                   experience_context)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class AuthorizationStatusDetails(object):

    """Implementation of the 'Authorization Status Details' model.

    The details of the authorized payment status.

    Attributes:
        reason (AuthorizationIncompleteReason): The reason why the authorized
            status is `PENDING`.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "reason": 'reason'
    }

    _optionals = [
        'reason',
    ]

    def __init__(self,
                 reason=APIHelper.SKIP):
        """Constructor for the AuthorizationStatusDetails class"""

        # Initialize members of the class
        if reason is not APIHelper.SKIP:
            self.reason = reason 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        reason = dictionary.get("reason") if dictionary.get("reason") else APIHelper.SKIP
        # Return an object of this model
        return cls(reason)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class PayPalWalletVaultInstruction(object):

    """Implementation of the 'PayPal Wallet Vault Instruction' model.

    TODO: type model description here.

    Attributes:
        store_in_vault (StoreInVaultInstruction): Defines how and when the
            payment source gets vaulted.
        description (str): The description displayed to PayPal consumer on the
            approval flow for PayPal, as well as on the PayPal payment token
            management experience on PayPal.com.
        usage_pattern (PayPalPaymentTokenUsagePattern): Expected
            business/pricing model for the billing agreement.
        usage_type (PayPalPaymentTokenUsageType): The usage type associated
            with the PayPal payment token.
        customer_type (PayPalPaymentTokenCustomerType): The customer type
            associated with the PayPal payment token. This is to indicate
            whether the customer acting on the merchant / platform is either a
            business or a consumer.
        permit_multiple_payment_tokens (bool): Create multiple payment tokens
            for the same payer, merchant/platform combination. Use this when
            the customer has not logged in at merchant/platform. The payment
            token thus generated, can then also be used to create the customer
            account at merchant/platform. Use this also when multiple payment
            tokens are required for the same payer, different customer at
            merchant/platform. This helps to identify customers distinctly
            even though they may share the same PayPal account. This only
            applies to PayPal payment source.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "usage_type": 'usage_type',
        "store_in_vault": 'store_in_vault',
        "description": 'description',
        "usage_pattern": 'usage_pattern',
        "customer_type": 'customer_type',
        "permit_multiple_payment_tokens": 'permit_multiple_payment_tokens'
    }

    _optionals = [
        'store_in_vault',
        'description',
        'usage_pattern',
        'customer_type',
        'permit_multiple_payment_tokens',
    ]

    def __init__(self,
                 usage_type=None,
                 store_in_vault=APIHelper.SKIP,
                 description=APIHelper.SKIP,
                 usage_pattern=APIHelper.SKIP,
                 customer_type='CONSUMER',
                 permit_multiple_payment_tokens=False):
        """Constructor for the PayPalWalletVaultInstruction class"""

        # Initialize members of the class
        if store_in_vault is not APIHelper.SKIP:
            self.store_in_vault = store_in_vault 
        if description is not APIHelper.SKIP:
            self.description = description 
        if usage_pattern is not APIHelper.SKIP:
            self.usage_pattern = usage_pattern 
        self.usage_type = usage_type 
        self.customer_type = customer_type 
        self.permit_multiple_payment_tokens = permit_multiple_payment_tokens 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        usage_type = dictionary.get("usage_type") if dictionary.get("usage_type") else None
        store_in_vault = dictionary.get("store_in_vault") if dictionary.get("store_in_vault") else APIHelper.SKIP
        description = dictionary.get("description") if dictionary.get("description") else APIHelper.SKIP
        usage_pattern = dictionary.get("usage_pattern") if dictionary.get("usage_pattern") else APIHelper.SKIP
        customer_type = dictionary.get("customer_type") if dictionary.get("customer_type") else 'CONSUMER'
        permit_multiple_payment_tokens = dictionary.get("permit_multiple_payment_tokens") if dictionary.get("permit_multiple_payment_tokens") else False
        # Return an object of this model
        return cls(usage_type,
                   store_in_vault,
                   description,
                   usage_pattern,
                   customer_type,
                   permit_multiple_payment_tokens)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class UniversalProductCode(object):

    """Implementation of the 'Universal Product Code' model.

    The Universal Product Code of the item.

    Attributes:
        mtype (UPCType): The Universal Product Code type.
        code (str): The UPC product code of the item.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "mtype": 'type',
        "code": 'code'
    }

    def __init__(self,
                 mtype=None,
                 code=None):
        """Constructor for the UniversalProductCode class"""

        # Initialize members of the class
        self.mtype = mtype 
        self.code = code 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        mtype = dictionary.get("type") if dictionary.get("type") else None
        code = dictionary.get("code") if dictionary.get("code") else None
        # Return an object of this model
        return cls(mtype,
                   code)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.three_d_secure_authentication_response import ThreeDSecureAuthenticationResponse


class AuthenticationResponse(object):

    """Implementation of the 'Authentication Response' model.

    Results of Authentication such as 3D Secure.

    Attributes:
        liability_shift (LiabilityShiftIndicator): Liability shift indicator.
            The outcome of the issuer's authentication.
        three_d_secure (ThreeDSecureAuthenticationResponse): Results of 3D
            Secure Authentication.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "liability_shift": 'liability_shift',
        "three_d_secure": 'three_d_secure'
    }

    _optionals = [
        'liability_shift',
        'three_d_secure',
    ]

    def __init__(self,
                 liability_shift=APIHelper.SKIP,
                 three_d_secure=APIHelper.SKIP):
        """Constructor for the AuthenticationResponse class"""

        # Initialize members of the class
        if liability_shift is not APIHelper.SKIP:
            self.liability_shift = liability_shift 
        if three_d_secure is not APIHelper.SKIP:
            self.three_d_secure = three_d_secure 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        liability_shift = dictionary.get("liability_shift") if dictionary.get("liability_shift") else APIHelper.SKIP
        three_d_secure = ThreeDSecureAuthenticationResponse.from_dictionary(dictionary.get('three_d_secure')) if 'three_d_secure' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(liability_shift,
                   three_d_secure)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.network_transaction_reference import NetworkTransactionReference


class CardStoredCredential(object):

    """Implementation of the 'Card Stored Credential' model.

    Provides additional details to process a payment using a `card` that has
    been stored or is intended to be stored (also referred to as
    stored_credential or card-on-file).<br/>Parameter
    compatibility:<br/><ul><li>`payment_type=ONE_TIME` is compatible only with
    `payment_initiator=CUSTOMER`.</li><li>`usage=FIRST` is compatible only
    with
    `payment_initiator=CUSTOMER`.</li><li>`previous_transaction_reference` or
    `previous_network_transaction_reference` is compatible only with
    `payment_initiator=MERCHANT`.</li><li>Only one of the parameters -
    `previous_transaction_reference` and
    `previous_network_transaction_reference` - can be present in the
    request.</li></ul>

    Attributes:
        payment_initiator (PaymentInitiator): The person or party who
            initiated or triggered the payment.
        payment_type (StoredPaymentSourcePaymentType): Indicates the type of
            the stored payment_source payment.
        usage (StoredPaymentSourceUsageType): Indicates if this is a `first`
            or `subsequent` payment using a stored payment source (also
            referred to as stored credential or card on file).
        previous_network_transaction_reference (NetworkTransactionReference):
            Reference values used by the card network to identify a
            transaction.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "payment_initiator": 'payment_initiator',
        "payment_type": 'payment_type',
        "usage": 'usage',
        "previous_network_transaction_reference": 'previous_network_transaction_reference'
    }

    _optionals = [
        'usage',
        'previous_network_transaction_reference',
    ]

    def __init__(self,
                 payment_initiator=None,
                 payment_type=None,
                 usage='DERIVED',
                 previous_network_transaction_reference=APIHelper.SKIP):
        """Constructor for the CardStoredCredential class"""

        # Initialize members of the class
        self.payment_initiator = payment_initiator 
        self.payment_type = payment_type 
        self.usage = usage 
        if previous_network_transaction_reference is not APIHelper.SKIP:
            self.previous_network_transaction_reference = previous_network_transaction_reference 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        payment_initiator = dictionary.get("payment_initiator") if dictionary.get("payment_initiator") else None
        payment_type = dictionary.get("payment_type") if dictionary.get("payment_type") else None
        usage = dictionary.get("usage") if dictionary.get("usage") else 'DERIVED'
        previous_network_transaction_reference = NetworkTransactionReference.from_dictionary(dictionary.get('previous_network_transaction_reference')) if 'previous_network_transaction_reference' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(payment_initiator,
                   payment_type,
                   usage,
                   previous_network_transaction_reference)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PayPalPaymentTokenUsagePattern(object):

    """Implementation of the 'PayPal Payment Token Usage Pattern' enum.

    Expected business/pricing model for the billing agreement.

    Attributes:
        IMMEDIATE: TODO: type description here.
        DEFERRED: TODO: type description here.
        RECURRING_PREPAID: TODO: type description here.
        RECURRING_POSTPAID: TODO: type description here.
        THRESHOLD_PREPAID: TODO: type description here.
        THRESHOLD_POSTPAID: TODO: type description here.

    """
    IMMEDIATE = 'IMMEDIATE'

    DEFERRED = 'DEFERRED'

    RECURRING_PREPAID = 'RECURRING_PREPAID'

    RECURRING_POSTPAID = 'RECURRING_POSTPAID'

    THRESHOLD_PREPAID = 'THRESHOLD_PREPAID'

    THRESHOLD_POSTPAID = 'THRESHOLD_POSTPAID'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money
from paypalserversdk.models.net_amount_breakdown_item import NetAmountBreakdownItem
from paypalserversdk.models.platform_fee import PlatformFee


class SellerPayableBreakdown(object):

    """Implementation of the 'Seller Payable Breakdown' model.

    The breakdown of the refund.

    Attributes:
        gross_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        paypal_fee (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        paypal_fee_in_receivable_currency (Money): The currency and amount for
            a financial transaction, such as a balance or payment due.
        net_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        net_amount_in_receivable_currency (Money): The currency and amount for
            a financial transaction, such as a balance or payment due.
        platform_fees (List[PlatformFee]): An array of platform or partner
            fees, commissions, or brokerage fees for the refund.
        net_amount_breakdown (List[NetAmountBreakdownItem]): An array of
            breakdown values for the net amount. Returned when the currency of
            the refund is different from the currency of the PayPal account
            where the payee holds their funds.
        total_refunded_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "gross_amount": 'gross_amount',
        "paypal_fee": 'paypal_fee',
        "paypal_fee_in_receivable_currency": 'paypal_fee_in_receivable_currency',
        "net_amount": 'net_amount',
        "net_amount_in_receivable_currency": 'net_amount_in_receivable_currency',
        "platform_fees": 'platform_fees',
        "net_amount_breakdown": 'net_amount_breakdown',
        "total_refunded_amount": 'total_refunded_amount'
    }

    _optionals = [
        'gross_amount',
        'paypal_fee',
        'paypal_fee_in_receivable_currency',
        'net_amount',
        'net_amount_in_receivable_currency',
        'platform_fees',
        'net_amount_breakdown',
        'total_refunded_amount',
    ]

    def __init__(self,
                 gross_amount=APIHelper.SKIP,
                 paypal_fee=APIHelper.SKIP,
                 paypal_fee_in_receivable_currency=APIHelper.SKIP,
                 net_amount=APIHelper.SKIP,
                 net_amount_in_receivable_currency=APIHelper.SKIP,
                 platform_fees=APIHelper.SKIP,
                 net_amount_breakdown=APIHelper.SKIP,
                 total_refunded_amount=APIHelper.SKIP):
        """Constructor for the SellerPayableBreakdown class"""

        # Initialize members of the class
        if gross_amount is not APIHelper.SKIP:
            self.gross_amount = gross_amount 
        if paypal_fee is not APIHelper.SKIP:
            self.paypal_fee = paypal_fee 
        if paypal_fee_in_receivable_currency is not APIHelper.SKIP:
            self.paypal_fee_in_receivable_currency = paypal_fee_in_receivable_currency 
        if net_amount is not APIHelper.SKIP:
            self.net_amount = net_amount 
        if net_amount_in_receivable_currency is not APIHelper.SKIP:
            self.net_amount_in_receivable_currency = net_amount_in_receivable_currency 
        if platform_fees is not APIHelper.SKIP:
            self.platform_fees = platform_fees 
        if net_amount_breakdown is not APIHelper.SKIP:
            self.net_amount_breakdown = net_amount_breakdown 
        if total_refunded_amount is not APIHelper.SKIP:
            self.total_refunded_amount = total_refunded_amount 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        gross_amount = Money.from_dictionary(dictionary.get('gross_amount')) if 'gross_amount' in dictionary.keys() else APIHelper.SKIP
        paypal_fee = Money.from_dictionary(dictionary.get('paypal_fee')) if 'paypal_fee' in dictionary.keys() else APIHelper.SKIP
        paypal_fee_in_receivable_currency = Money.from_dictionary(dictionary.get('paypal_fee_in_receivable_currency')) if 'paypal_fee_in_receivable_currency' in dictionary.keys() else APIHelper.SKIP
        net_amount = Money.from_dictionary(dictionary.get('net_amount')) if 'net_amount' in dictionary.keys() else APIHelper.SKIP
        net_amount_in_receivable_currency = Money.from_dictionary(dictionary.get('net_amount_in_receivable_currency')) if 'net_amount_in_receivable_currency' in dictionary.keys() else APIHelper.SKIP
        platform_fees = None
        if dictionary.get('platform_fees') is not None:
            platform_fees = [PlatformFee.from_dictionary(x) for x in dictionary.get('platform_fees')]
        else:
            platform_fees = APIHelper.SKIP
        net_amount_breakdown = None
        if dictionary.get('net_amount_breakdown') is not None:
            net_amount_breakdown = [NetAmountBreakdownItem.from_dictionary(x) for x in dictionary.get('net_amount_breakdown')]
        else:
            net_amount_breakdown = APIHelper.SKIP
        total_refunded_amount = Money.from_dictionary(dictionary.get('total_refunded_amount')) if 'total_refunded_amount' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(gross_amount,
                   paypal_fee,
                   paypal_fee_in_receivable_currency,
                   net_amount,
                   net_amount_in_receivable_currency,
                   platform_fees,
                   net_amount_breakdown,
                   total_refunded_amount)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class VaultVenmoExperienceContext(object):

    """Implementation of the 'Vault Venmo Experience Context' model.

    Customizes the Vault creation flow experience for your customers.

    Attributes:
        brand_name (str): The label that overrides the business name in the
            PayPal account on the PayPal site. The pattern is defined by an
            external party and supports Unicode.
        shipping_preference (str): The shipping preference. This only applies
            to PayPal payment source.
        vault_instruction (str): Vault Instruction on action to be performed
            after a successful payer approval.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "brand_name": 'brand_name',
        "shipping_preference": 'shipping_preference',
        "vault_instruction": 'vault_instruction'
    }

    _optionals = [
        'brand_name',
        'shipping_preference',
        'vault_instruction',
    ]

    def __init__(self,
                 brand_name=APIHelper.SKIP,
                 shipping_preference='GET_FROM_FILE',
                 vault_instruction='ON_CREATE_PAYMENT_TOKENS'):
        """Constructor for the VaultVenmoExperienceContext class"""

        # Initialize members of the class
        if brand_name is not APIHelper.SKIP:
            self.brand_name = brand_name 
        self.shipping_preference = shipping_preference 
        self.vault_instruction = vault_instruction 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        brand_name = dictionary.get("brand_name") if dictionary.get("brand_name") else APIHelper.SKIP
        shipping_preference = dictionary.get("shipping_preference") if dictionary.get("shipping_preference") else 'GET_FROM_FILE'
        vault_instruction = dictionary.get("vault_instruction") if dictionary.get("vault_instruction") else 'ON_CREATE_PAYMENT_TOKENS'
        # Return an object of this model
        return cls(brand_name,
                   shipping_preference,
                   vault_instruction)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class ShippingName(object):

    """Implementation of the 'Shipping Name' model.

    The name of the party.

    Attributes:
        full_name (str): When the party is a person, the party's full name.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "full_name": 'full_name'
    }

    _optionals = [
        'full_name',
    ]

    def __init__(self,
                 full_name=APIHelper.SKIP):
        """Constructor for the ShippingName class"""

        # Initialize members of the class
        if full_name is not APIHelper.SKIP:
            self.full_name = full_name 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        full_name = dictionary.get("full_name") if dictionary.get("full_name") else APIHelper.SKIP
        # Return an object of this model
        return cls(full_name)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.address import Address
from paypalserversdk.models.line_item import LineItem
from paypalserversdk.models.money import Money


class Level3CardProcessingData(object):

    """Implementation of the 'Level 3 Card Processing Data' model.

    The level 3 card processing data collections, If your merchant account has
    been configured for Level 3 processing this field will be passed to the
    processor on your behalf. Please contact your PayPal Technical Account
    Manager to define level 3 data for your business.

    Attributes:
        shipping_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        duty_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        discount_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        shipping_address (Address): The portable international postal address.
            Maps to
            [AddressValidationMetadata](https://github.com/googlei18n/libaddres
            sinput/wiki/AddressValidationMetadata) and HTML 5.1 [Autofilling
            form controls: the autocomplete
            attribute](https://www.w3.org/TR/html51/sec-forms.html#autofilling-
            form-controls-the-autocomplete-attribute).
        ships_from_postal_code (str): Use this field to specify the postal
            code of the shipping location.
        line_items (List[LineItem]): A list of the items that were purchased
            with this payment. If your merchant account has been configured
            for Level 3 processing this field will be passed to the processor
            on your behalf.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "shipping_amount": 'shipping_amount',
        "duty_amount": 'duty_amount',
        "discount_amount": 'discount_amount',
        "shipping_address": 'shipping_address',
        "ships_from_postal_code": 'ships_from_postal_code',
        "line_items": 'line_items'
    }

    _optionals = [
        'shipping_amount',
        'duty_amount',
        'discount_amount',
        'shipping_address',
        'ships_from_postal_code',
        'line_items',
    ]

    def __init__(self,
                 shipping_amount=APIHelper.SKIP,
                 duty_amount=APIHelper.SKIP,
                 discount_amount=APIHelper.SKIP,
                 shipping_address=APIHelper.SKIP,
                 ships_from_postal_code=APIHelper.SKIP,
                 line_items=APIHelper.SKIP):
        """Constructor for the Level3CardProcessingData class"""

        # Initialize members of the class
        if shipping_amount is not APIHelper.SKIP:
            self.shipping_amount = shipping_amount 
        if duty_amount is not APIHelper.SKIP:
            self.duty_amount = duty_amount 
        if discount_amount is not APIHelper.SKIP:
            self.discount_amount = discount_amount 
        if shipping_address is not APIHelper.SKIP:
            self.shipping_address = shipping_address 
        if ships_from_postal_code is not APIHelper.SKIP:
            self.ships_from_postal_code = ships_from_postal_code 
        if line_items is not APIHelper.SKIP:
            self.line_items = line_items 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        shipping_amount = Money.from_dictionary(dictionary.get('shipping_amount')) if 'shipping_amount' in dictionary.keys() else APIHelper.SKIP
        duty_amount = Money.from_dictionary(dictionary.get('duty_amount')) if 'duty_amount' in dictionary.keys() else APIHelper.SKIP
        discount_amount = Money.from_dictionary(dictionary.get('discount_amount')) if 'discount_amount' in dictionary.keys() else APIHelper.SKIP
        shipping_address = Address.from_dictionary(dictionary.get('shipping_address')) if 'shipping_address' in dictionary.keys() else APIHelper.SKIP
        ships_from_postal_code = dictionary.get("ships_from_postal_code") if dictionary.get("ships_from_postal_code") else APIHelper.SKIP
        line_items = None
        if dictionary.get('line_items') is not None:
            line_items = [LineItem.from_dictionary(x) for x in dictionary.get('line_items')]
        else:
            line_items = APIHelper.SKIP
        # Return an object of this model
        return cls(shipping_amount,
                   duty_amount,
                   discount_amount,
                   shipping_address,
                   ships_from_postal_code,
                   line_items)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class PayerBase(object):

    """Implementation of the 'Payer Base' model.

    The customer who approves and pays for the order. The customer is also
    known as the payer.

    Attributes:
        email_address (str): The internationalized email
            address.<blockquote><strong>Note:</strong> Up to 64 characters are
            allowed before and 255 characters are allowed after the
            <code>@</code> sign. However, the generally accepted maximum
            length for an email address is 254 characters. The pattern
            verifies that an unquoted <code>@</code> sign
            exists.</blockquote>
        payer_id (str): The account identifier for a PayPal account.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "email_address": 'email_address',
        "payer_id": 'payer_id'
    }

    _optionals = [
        'email_address',
        'payer_id',
    ]

    def __init__(self,
                 email_address=APIHelper.SKIP,
                 payer_id=APIHelper.SKIP):
        """Constructor for the PayerBase class"""

        # Initialize members of the class
        if email_address is not APIHelper.SKIP:
            self.email_address = email_address 
        if payer_id is not APIHelper.SKIP:
            self.payer_id = payer_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        email_address = dictionary.get("email_address") if dictionary.get("email_address") else APIHelper.SKIP
        payer_id = dictionary.get("payer_id") if dictionary.get("payer_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(email_address,
                   payer_id)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class EnrollmentStatus(object):

    """Implementation of the 'Enrollment Status' enum.

    Status of Authentication eligibility.

    Attributes:
        Y: TODO: type description here.
        N: TODO: type description here.
        U: TODO: type description here.
        B: TODO: type description here.

    """
    Y = 'Y'

    N = 'N'

    U = 'U'

    B = 'B'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.capture_payment_instruction import CapturePaymentInstruction
from paypalserversdk.models.money import Money


class CaptureRequest(object):

    """Implementation of the 'Capture Request' model.

    TODO: type model description here.

    Attributes:
        invoice_id (str): The API caller-provided external invoice number for
            this order. Appears in both the payer's transaction history and
            the emails that the payer receives.
        note_to_payer (str): An informational note about this settlement.
            Appears in both the payer's transaction history and the emails
            that the payer receives.
        amount (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.
        final_capture (bool): Indicates whether you can make additional
            captures against the authorized payment. Set to `true` if you do
            not intend to capture additional payments against the
            authorization. Set to `false` if you intend to capture additional
            payments against the authorization.
        payment_instruction (CapturePaymentInstruction): Any additional
            payment instructions to be consider during payment processing.
            This processing instruction is applicable for Capturing an order
            or Authorizing an Order.
        soft_descriptor (str): The payment descriptor on the payer's account
            statement.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "invoice_id": 'invoice_id',
        "note_to_payer": 'note_to_payer',
        "amount": 'amount',
        "final_capture": 'final_capture',
        "payment_instruction": 'payment_instruction',
        "soft_descriptor": 'soft_descriptor'
    }

    _optionals = [
        'invoice_id',
        'note_to_payer',
        'amount',
        'final_capture',
        'payment_instruction',
        'soft_descriptor',
    ]

    def __init__(self,
                 invoice_id=APIHelper.SKIP,
                 note_to_payer=APIHelper.SKIP,
                 amount=APIHelper.SKIP,
                 final_capture=False,
                 payment_instruction=APIHelper.SKIP,
                 soft_descriptor=APIHelper.SKIP):
        """Constructor for the CaptureRequest class"""

        # Initialize members of the class
        if invoice_id is not APIHelper.SKIP:
            self.invoice_id = invoice_id 
        if note_to_payer is not APIHelper.SKIP:
            self.note_to_payer = note_to_payer 
        if amount is not APIHelper.SKIP:
            self.amount = amount 
        self.final_capture = final_capture 
        if payment_instruction is not APIHelper.SKIP:
            self.payment_instruction = payment_instruction 
        if soft_descriptor is not APIHelper.SKIP:
            self.soft_descriptor = soft_descriptor 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        invoice_id = dictionary.get("invoice_id") if dictionary.get("invoice_id") else APIHelper.SKIP
        note_to_payer = dictionary.get("note_to_payer") if dictionary.get("note_to_payer") else APIHelper.SKIP
        amount = Money.from_dictionary(dictionary.get('amount')) if 'amount' in dictionary.keys() else APIHelper.SKIP
        final_capture = dictionary.get("final_capture") if dictionary.get("final_capture") else False
        payment_instruction = CapturePaymentInstruction.from_dictionary(dictionary.get('payment_instruction')) if 'payment_instruction' in dictionary.keys() else APIHelper.SKIP
        soft_descriptor = dictionary.get("soft_descriptor") if dictionary.get("soft_descriptor") else APIHelper.SKIP
        # Return an object of this model
        return cls(invoice_id,
                   note_to_payer,
                   amount,
                   final_capture,
                   payment_instruction,
                   soft_descriptor)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class CardFromRequest(object):

    """Implementation of the 'Card From Request' model.

    Representation of card details as received in the request.

    Attributes:
        expiry (str): The year and month, in ISO-8601 `YYYY-MM` date format.
            See [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6).
        last_digits (str): The last digits of the payment card.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "expiry": 'expiry',
        "last_digits": 'last_digits'
    }

    _optionals = [
        'expiry',
        'last_digits',
    ]

    def __init__(self,
                 expiry=APIHelper.SKIP,
                 last_digits=APIHelper.SKIP):
        """Constructor for the CardFromRequest class"""

        # Initialize members of the class
        if expiry is not APIHelper.SKIP:
            self.expiry = expiry 
        if last_digits is not APIHelper.SKIP:
            self.last_digits = last_digits 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        expiry = dictionary.get("expiry") if dictionary.get("expiry") else APIHelper.SKIP
        last_digits = dictionary.get("last_digits") if dictionary.get("last_digits") else APIHelper.SKIP
        # Return an object of this model
        return cls(expiry,
                   last_digits)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class RelatedIdentifiers(object):

    """Implementation of the 'Related Identifiers' model.

    Identifiers related to a specific resource.

    Attributes:
        order_id (str): Order ID related to the resource.
        authorization_id (str): Authorization ID related to the resource.
        capture_id (str): Capture ID related to the resource.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "order_id": 'order_id',
        "authorization_id": 'authorization_id',
        "capture_id": 'capture_id'
    }

    _optionals = [
        'order_id',
        'authorization_id',
        'capture_id',
    ]

    def __init__(self,
                 order_id=APIHelper.SKIP,
                 authorization_id=APIHelper.SKIP,
                 capture_id=APIHelper.SKIP):
        """Constructor for the RelatedIdentifiers class"""

        # Initialize members of the class
        if order_id is not APIHelper.SKIP:
            self.order_id = order_id 
        if authorization_id is not APIHelper.SKIP:
            self.authorization_id = authorization_id 
        if capture_id is not APIHelper.SKIP:
            self.capture_id = capture_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        order_id = dictionary.get("order_id") if dictionary.get("order_id") else APIHelper.SKIP
        authorization_id = dictionary.get("authorization_id") if dictionary.get("authorization_id") else APIHelper.SKIP
        capture_id = dictionary.get("capture_id") if dictionary.get("capture_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(order_id,
                   authorization_id,
                   capture_id)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class DisbursementMode(object):

    """Implementation of the 'Disbursement Mode' enum.

    The funds that are held on behalf of the merchant.

    Attributes:
        INSTANT: TODO: type description here.
        DELAYED: TODO: type description here.

    """
    INSTANT = 'INSTANT'

    DELAYED = 'DELAYED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.authentication_response import AuthenticationResponse
from paypalserversdk.models.bin_details import BinDetails
from paypalserversdk.models.card_attributes_response import CardAttributesResponse
from paypalserversdk.models.card_from_request import CardFromRequest


class CardResponse(object):

    """Implementation of the 'Card Response' model.

    The payment card to use to fund a payment. Card can be a credit or debit
    card.

    Attributes:
        name (str): The card holder's name as it appears on the card.
        last_digits (str): The last digits of the payment card.
        brand (CardBrand): The card network or brand. Applies to credit,
            debit, gift, and payment cards.
        available_networks (List[CardBrand]): Array of brands or networks
            associated with the card.
        mtype (CardType): Type of card. i.e Credit, Debit and so on.
        authentication_result (AuthenticationResponse): Results of
            Authentication such as 3D Secure.
        attributes (CardAttributesResponse): Additional attributes associated
            with the use of this card.
        from_request (CardFromRequest): Representation of card details as
            received in the request.
        expiry (str): The year and month, in ISO-8601 `YYYY-MM` date format.
            See [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6).
        bin_details (BinDetails): Bank Identification Number (BIN) details
            used to fund a payment.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "last_digits": 'last_digits',
        "brand": 'brand',
        "available_networks": 'available_networks',
        "mtype": 'type',
        "authentication_result": 'authentication_result',
        "attributes": 'attributes',
        "from_request": 'from_request',
        "expiry": 'expiry',
        "bin_details": 'bin_details'
    }

    _optionals = [
        'name',
        'last_digits',
        'brand',
        'available_networks',
        'mtype',
        'authentication_result',
        'attributes',
        'from_request',
        'expiry',
        'bin_details',
    ]

    def __init__(self,
                 name=APIHelper.SKIP,
                 last_digits=APIHelper.SKIP,
                 brand=APIHelper.SKIP,
                 available_networks=APIHelper.SKIP,
                 mtype=APIHelper.SKIP,
                 authentication_result=APIHelper.SKIP,
                 attributes=APIHelper.SKIP,
                 from_request=APIHelper.SKIP,
                 expiry=APIHelper.SKIP,
                 bin_details=APIHelper.SKIP):
        """Constructor for the CardResponse class"""

        # Initialize members of the class
        if name is not APIHelper.SKIP:
            self.name = name 
        if last_digits is not APIHelper.SKIP:
            self.last_digits = last_digits 
        if brand is not APIHelper.SKIP:
            self.brand = brand 
        if available_networks is not APIHelper.SKIP:
            self.available_networks = available_networks 
        if mtype is not APIHelper.SKIP:
            self.mtype = mtype 
        if authentication_result is not APIHelper.SKIP:
            self.authentication_result = authentication_result 
        if attributes is not APIHelper.SKIP:
            self.attributes = attributes 
        if from_request is not APIHelper.SKIP:
            self.from_request = from_request 
        if expiry is not APIHelper.SKIP:
            self.expiry = expiry 
        if bin_details is not APIHelper.SKIP:
            self.bin_details = bin_details 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        last_digits = dictionary.get("last_digits") if dictionary.get("last_digits") else APIHelper.SKIP
        brand = dictionary.get("brand") if dictionary.get("brand") else APIHelper.SKIP
        available_networks = dictionary.get("available_networks") if dictionary.get("available_networks") else APIHelper.SKIP
        mtype = dictionary.get("type") if dictionary.get("type") else APIHelper.SKIP
        authentication_result = AuthenticationResponse.from_dictionary(dictionary.get('authentication_result')) if 'authentication_result' in dictionary.keys() else APIHelper.SKIP
        attributes = CardAttributesResponse.from_dictionary(dictionary.get('attributes')) if 'attributes' in dictionary.keys() else APIHelper.SKIP
        from_request = CardFromRequest.from_dictionary(dictionary.get('from_request')) if 'from_request' in dictionary.keys() else APIHelper.SKIP
        expiry = dictionary.get("expiry") if dictionary.get("expiry") else APIHelper.SKIP
        bin_details = BinDetails.from_dictionary(dictionary.get('bin_details')) if 'bin_details' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   last_digits,
                   brand,
                   available_networks,
                   mtype,
                   authentication_result,
                   attributes,
                   from_request,
                   expiry,
                   bin_details)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CardVerificationMethod(object):

    """Implementation of the 'Card Verification Method' enum.

    The method used for card verification.

    Attributes:
        SCA_ALWAYS: TODO: type description here.
        SCA_WHEN_REQUIRED: TODO: type description here.
        ENUM_3D_SECURE: TODO: type description here.
        AVS_CVV: TODO: type description here.

    """
    SCA_ALWAYS = 'SCA_ALWAYS'

    SCA_WHEN_REQUIRED = 'SCA_WHEN_REQUIRED'

    ENUM_3D_SECURE = '3D_SECURE'

    AVS_CVV = 'AVS_CVV'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from paypalserversdk.api_helper import APIHelper
import paypalserversdk.exceptions.api_exception
from paypalserversdk.models.error_details import ErrorDetails
from paypalserversdk.models.link_description import LinkDescription


class ErrorException(paypalserversdk.exceptions.api_exception.APIException):
    def __init__(self, reason, response):
        """Constructor for the ErrorException class

        Args:
            reason (string): The reason (or error message) for the Exception
                to be raised.
            response (HttpResponse): The HttpResponse of the API call.

        """
        super(ErrorException, self).__init__(reason, response)
        dictionary = APIHelper.json_deserialize(self.response.text)
        if isinstance(dictionary, dict):
            self.unbox(dictionary)

    def unbox(self, dictionary):
        """Populates the properties of this object by extracting them from a dictionary.

        Args:
            dictionary (dictionary): A dictionary representation of the object as
            obtained from the deserialization of the server's response. The keys
            MUST match property names in the API description.

        """
        self.name = dictionary.get("name") if dictionary.get("name") else None
        self.message = dictionary.get("message") if dictionary.get("message") else None
        self.debug_id = dictionary.get("debug_id") if dictionary.get("debug_id") else None
        self.details = None
        if dictionary.get('details') is not None:
            self.details = [ErrorDetails.from_dictionary(x) for x in dictionary.get('details')]
        else:
            self.details = None
        self.links = None
        if dictionary.get('links') is not None:
            self.links = [LinkDescription.from_dictionary(x) for x in dictionary.get('links')]
        else:
            self.links = None



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class BLIKOneClickPaymentRequest(object):

    """Implementation of the 'BLIK One-Click Payment Request' model.

    Information used to pay using BLIK one-click flow.

    Attributes:
        auth_code (str): The 6-digit code used to authenticate a consumer
            within BLIK.
        consumer_reference (str): The merchant generated, unique reference
            serving as a primary identifier for accounts connected between
            Blik and a merchant.
        alias_label (str): A bank defined identifier used as a display name to
            allow the payer to differentiate between multiple registered bank
            accounts.
        alias_key (str): A Blik-defined identifier for a specific Blik-enabled
            bank account that is associated with a given merchant. Used only
            in conjunction with a Consumer Reference.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "consumer_reference": 'consumer_reference',
        "auth_code": 'auth_code',
        "alias_label": 'alias_label',
        "alias_key": 'alias_key'
    }

    _optionals = [
        'auth_code',
        'alias_label',
        'alias_key',
    ]

    def __init__(self,
                 consumer_reference=None,
                 auth_code=APIHelper.SKIP,
                 alias_label=APIHelper.SKIP,
                 alias_key=APIHelper.SKIP):
        """Constructor for the BLIKOneClickPaymentRequest class"""

        # Initialize members of the class
        if auth_code is not APIHelper.SKIP:
            self.auth_code = auth_code 
        self.consumer_reference = consumer_reference 
        if alias_label is not APIHelper.SKIP:
            self.alias_label = alias_label 
        if alias_key is not APIHelper.SKIP:
            self.alias_key = alias_key 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        consumer_reference = dictionary.get("consumer_reference") if dictionary.get("consumer_reference") else None
        auth_code = dictionary.get("auth_code") if dictionary.get("auth_code") else APIHelper.SKIP
        alias_label = dictionary.get("alias_label") if dictionary.get("alias_label") else APIHelper.SKIP
        alias_key = dictionary.get("alias_key") if dictionary.get("alias_key") else APIHelper.SKIP
        # Return an object of this model
        return cls(consumer_reference,
                   auth_code,
                   alias_label,
                   alias_key)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ItemCategory(object):

    """Implementation of the 'Item Category' enum.

    The item category type.

    Attributes:
        DIGITAL_GOODS: TODO: type description here.
        PHYSICAL_GOODS: TODO: type description here.
        DONATION: TODO: type description here.

    """
    DIGITAL_GOODS = 'DIGITAL_GOODS'

    PHYSICAL_GOODS = 'PHYSICAL_GOODS'

    DONATION = 'DONATION'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.payment_method_preference import PaymentMethodPreference
from paypalserversdk.models.stored_payment_source import StoredPaymentSource


class OrderApplicationContext(object):

    """Implementation of the 'Order Application Context' model.

    Customizes the payer experience during the approval process for the
    payment with PayPal.<blockquote><strong>Note:</strong> Partners and
    Marketplaces might configure <code>brand_name</code> and
    <code>shipping_preference</code> during partner account setup, which
    overrides the request values.</blockquote>

    Attributes:
        brand_name (str): DEPRECATED. The label that overrides the business
            name in the PayPal account on the PayPal site. The fields in
            `application_context` are now available in the
            `experience_context` object under the `payment_source` which
            supports them (eg.
            `payment_source.paypal.experience_context.brand_name`). Please
            specify this field in the `experience_context` object instead of
            the `application_context` object.
        locale (str): The [language
            tag](https://tools.ietf.org/html/bcp47#section-2) for the language
            in which to localize the error-related strings, such as messages,
            issues, and suggested actions. The tag is made up of the [ISO
            639-2 language
            code](https://www.loc.gov/standards/iso639-2/php/code_list.php),
            the optional [ISO-15924 script
            tag](https://www.unicode.org/iso15924/codelists.html), and the
            [ISO-3166 alpha-2 country
            code](/api/rest/reference/country-codes/) or [M49 region
            code](https://unstats.un.org/unsd/methodology/m49/).
        landing_page (OrderApplicationContextLandingPage): DEPRECATED.
            DEPRECATED. The type of landing page to show on the PayPal site
            for customer checkout.  The fields in `application_context` are
            now available in the `experience_context` object under the
            `payment_source` which supports them (eg.
            `payment_source.paypal.experience_context.landing_page`). Please
            specify this field in the `experience_context` object instead of
            the `application_context` object.
        shipping_preference (OrderApplicationContextShippingPreference):
            DEPRECATED. DEPRECATED. The shipping preference:<ul><li>Displays
            the shipping address to the customer.</li><li>Enables the customer
            to choose an address on the PayPal site.</li><li>Restricts the
            customer from changing the address during the payment-approval
            process.</li></ul>.  The fields in `application_context` are now
            available in the `experience_context` object under the
            `payment_source` which supports them (eg.
            `payment_source.paypal.experience_context.shipping_preference`).
            Please specify this field in the `experience_context` object
            instead of the `application_context` object.
        user_action (OrderApplicationContextUserAction): DEPRECATED.
            Configures a <strong>Continue</strong> or <strong>Pay Now</strong>
            checkout flow.  The fields in `application_context` are now
            available in the `experience_context` object under the
            `payment_source` which supports them (eg.
            `payment_source.paypal.experience_context.user_action`). Please
            specify this field in the `experience_context` object instead of
            the `application_context` object.
        payment_method (PaymentMethodPreference): The customer and merchant
            payment preferences.
        return_url (str): DEPRECATED. The URL where the customer is redirected
            after the customer approves the payment. The fields in
            `application_context` are now available in the
            `experience_context` object under the `payment_source` which
            supports them (eg.
            `payment_source.paypal.experience_context.return_url`). Please
            specify this field in the `experience_context` object instead of
            the `application_context` object.
        cancel_url (str): DEPRECATED. The URL where the customer is redirected
            after the customer cancels the payment. The fields in
            `application_context` are now available in the
            `experience_context` object under the `payment_source` which
            supports them (eg.
            `payment_source.paypal.experience_context.cancel_url`). Please
            specify this field in the `experience_context` object instead of
            the `application_context` object.
        stored_payment_source (StoredPaymentSource): Provides additional
            details to process a payment using a `payment_source` that has
            been stored or is intended to be stored (also referred to as
            stored_credential or card-on-file).<br/>Parameter
            compatibility:<br/><ul><li>`payment_type=ONE_TIME` is compatible
            only with `payment_initiator=CUSTOMER`.</li><li>`usage=FIRST` is
            compatible only with
            `payment_initiator=CUSTOMER`.</li><li>`previous_transaction_referen
            ce` or `previous_network_transaction_reference` is compatible only
            with `payment_initiator=MERCHANT`.</li><li>Only one of the
            parameters - `previous_transaction_reference` and
            `previous_network_transaction_reference` - can be present in the
            request.</li></ul>

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "brand_name": 'brand_name',
        "locale": 'locale',
        "landing_page": 'landing_page',
        "shipping_preference": 'shipping_preference',
        "user_action": 'user_action',
        "payment_method": 'payment_method',
        "return_url": 'return_url',
        "cancel_url": 'cancel_url',
        "stored_payment_source": 'stored_payment_source'
    }

    _optionals = [
        'brand_name',
        'locale',
        'landing_page',
        'shipping_preference',
        'user_action',
        'payment_method',
        'return_url',
        'cancel_url',
        'stored_payment_source',
    ]

    def __init__(self,
                 brand_name=APIHelper.SKIP,
                 locale=APIHelper.SKIP,
                 landing_page='NO_PREFERENCE',
                 shipping_preference='GET_FROM_FILE',
                 user_action='CONTINUE',
                 payment_method=APIHelper.SKIP,
                 return_url=APIHelper.SKIP,
                 cancel_url=APIHelper.SKIP,
                 stored_payment_source=APIHelper.SKIP):
        """Constructor for the OrderApplicationContext class"""

        # Initialize members of the class
        if brand_name is not APIHelper.SKIP:
            self.brand_name = brand_name 
        if locale is not APIHelper.SKIP:
            self.locale = locale 
        self.landing_page = landing_page 
        self.shipping_preference = shipping_preference 
        self.user_action = user_action 
        if payment_method is not APIHelper.SKIP:
            self.payment_method = payment_method 
        if return_url is not APIHelper.SKIP:
            self.return_url = return_url 
        if cancel_url is not APIHelper.SKIP:
            self.cancel_url = cancel_url 
        if stored_payment_source is not APIHelper.SKIP:
            self.stored_payment_source = stored_payment_source 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        brand_name = dictionary.get("brand_name") if dictionary.get("brand_name") else APIHelper.SKIP
        locale = dictionary.get("locale") if dictionary.get("locale") else APIHelper.SKIP
        landing_page = dictionary.get("landing_page") if dictionary.get("landing_page") else 'NO_PREFERENCE'
        shipping_preference = dictionary.get("shipping_preference") if dictionary.get("shipping_preference") else 'GET_FROM_FILE'
        user_action = dictionary.get("user_action") if dictionary.get("user_action") else 'CONTINUE'
        payment_method = PaymentMethodPreference.from_dictionary(dictionary.get('payment_method')) if 'payment_method' in dictionary.keys() else APIHelper.SKIP
        return_url = dictionary.get("return_url") if dictionary.get("return_url") else APIHelper.SKIP
        cancel_url = dictionary.get("cancel_url") if dictionary.get("cancel_url") else APIHelper.SKIP
        stored_payment_source = StoredPaymentSource.from_dictionary(dictionary.get('stored_payment_source')) if 'stored_payment_source' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(brand_name,
                   locale,
                   landing_page,
                   shipping_preference,
                   user_action,
                   payment_method,
                   return_url,
                   cancel_url,
                   stored_payment_source)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class ActivityTimestamps(object):

    """Implementation of the 'Activity Timestamps' model.

    The date and time stamps that are common to authorized payment, captured
    payment, and refund transactions.

    Attributes:
        create_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are
            optional.<blockquote><strong>Note:</strong> The regular expression
            provides guidance but does not reject all invalid
            dates.</blockquote>
        update_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are
            optional.<blockquote><strong>Note:</strong> The regular expression
            provides guidance but does not reject all invalid
            dates.</blockquote>

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "create_time": 'create_time',
        "update_time": 'update_time'
    }

    _optionals = [
        'create_time',
        'update_time',
    ]

    def __init__(self,
                 create_time=APIHelper.SKIP,
                 update_time=APIHelper.SKIP):
        """Constructor for the ActivityTimestamps class"""

        # Initialize members of the class
        if create_time is not APIHelper.SKIP:
            self.create_time = create_time 
        if update_time is not APIHelper.SKIP:
            self.update_time = update_time 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        create_time = dictionary.get("create_time") if dictionary.get("create_time") else APIHelper.SKIP
        update_time = dictionary.get("update_time") if dictionary.get("update_time") else APIHelper.SKIP
        # Return an object of this model
        return cls(create_time,
                   update_time)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CardBrand(object):

    """Implementation of the 'Card Brand' enum.

    The card network or brand. Applies to credit, debit, gift, and payment
    cards.

    Attributes:
        VISA: TODO: type description here.
        MASTERCARD: TODO: type description here.
        DISCOVER: TODO: type description here.
        AMEX: TODO: type description here.
        SOLO: TODO: type description here.
        JCB: TODO: type description here.
        STAR: TODO: type description here.
        DELTA: TODO: type description here.
        SWITCH: TODO: type description here.
        MAESTRO: TODO: type description here.
        CB_NATIONALE: TODO: type description here.
        CONFIGOGA: TODO: type description here.
        CONFIDIS: TODO: type description here.
        ELECTRON: TODO: type description here.
        CETELEM: TODO: type description here.
        CHINA_UNION_PAY: TODO: type description here.
        DINERS: TODO: type description here.
        ELO: TODO: type description here.
        HIPER: TODO: type description here.
        HIPERCARD: TODO: type description here.
        RUPAY: TODO: type description here.
        GE: TODO: type description here.
        SYNCHRONY: TODO: type description here.
        EFTPOS: TODO: type description here.
        UNKNOWN: TODO: type description here.

    """
    VISA = 'VISA'

    MASTERCARD = 'MASTERCARD'

    DISCOVER = 'DISCOVER'

    AMEX = 'AMEX'

    SOLO = 'SOLO'

    JCB = 'JCB'

    STAR = 'STAR'

    DELTA = 'DELTA'

    SWITCH = 'SWITCH'

    MAESTRO = 'MAESTRO'

    CB_NATIONALE = 'CB_NATIONALE'

    CONFIGOGA = 'CONFIGOGA'

    CONFIDIS = 'CONFIDIS'

    ELECTRON = 'ELECTRON'

    CETELEM = 'CETELEM'

    CHINA_UNION_PAY = 'CHINA_UNION_PAY'

    DINERS = 'DINERS'

    ELO = 'ELO'

    HIPER = 'HIPER'

    HIPERCARD = 'HIPERCARD'

    RUPAY = 'RUPAY'

    GE = 'GE'

    SYNCHRONY = 'SYNCHRONY'

    EFTPOS = 'EFTPOS'

    UNKNOWN = 'UNKNOWN'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from paypalserversdk.api_helper import APIHelper
from paypalserversdk.configuration import Server
from paypalserversdk.http.api_response import ApiResponse
from paypalserversdk.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from paypalserversdk.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from paypalserversdk.models.order import Order
from paypalserversdk.models.order_authorize_response import OrderAuthorizeResponse
from paypalserversdk.exceptions.error_exception import ErrorException


class OrdersController(BaseController):

    """A Controller to access Endpoints in the paypalserversdk API."""
    def __init__(self, config):
        super(OrdersController, self).__init__(config)

    def orders_create(self,
                      options=dict()):
        """Does a POST request to /v2/checkout/orders.

        Creates an order. Merchants and partners can add Level 2 and 3 data to
        payments to reduce risk and payment processing costs. For more
        information about processing payments, see <a
        href="https://developer.paypal.com/docs/checkout/advanced/processing/">
        checkout</a> or <a
        href="https://developer.paypal.com/docs/multiparty/checkout/advanced/pr
        ocessing/">multiparty checkout</a>.<blockquote><strong>Note:</strong>
        For error handling and troubleshooting, see <a
        href="https://developer.paypal.com/api/rest/reference/orders/v2/errors/
        #create-order">Orders v2 errors</a>.</blockquote>

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    body -- OrderRequest -- TODO: type description here.
                    pay_pal_request_id -- str -- The server stores keys for 6
                        hours. The API callers can request the times to up to
                        72 hours by speaking to their Account Manager.
                    pay_pal_partner_attribution_id -- str -- TODO: type
                        description here.
                    pay_pal_client_metadata_id -- str -- TODO: type
                        description here.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value
                        is:<ul><li><code>return=minimal</code>. The server
                        returns a minimal response to optimize communication
                        between the API caller and the server. A minimal
                        response includes the <code>id</code>,
                        <code>status</code> and HATEOAS
                        links.</li><li><code>return=representation</code>. The
                        server returns a complete resource representation,
                        including the current state of the
                        resource.</li></ul>

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful response to an idempotent request returns the HTTP
                `200 OK` status code with a JSON response body that shows
                order details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('pay_pal_request_id', None)))
            .header_param(Parameter()
                          .key('PayPal-Partner-Attribution-Id')
                          .value(options.get('pay_pal_partner_attribution_id', None)))
            .header_param(Parameter()
                          .key('PayPal-Client-Metadata-Id')
                          .value(options.get('pay_pal_client_metadata_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Order.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def orders_get(self,
                   options=dict()):
        """Does a GET request to /v2/checkout/orders/{id}.

        Shows details for an order, by ID.<blockquote><strong>Note:</strong>
        For error handling and troubleshooting, see <a
        href="https://developer.paypal.com/api/rest/reference/orders/v2/errors/
        #get-order">Orders v2 errors</a>.</blockquote>

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order for which to show
                        details.
                    fields -- str -- A comma-separated list of fields that
                        should be returned for the order. Valid filter field
                        is `payment_source`.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `200 OK` status code and a
                JSON response body that shows order details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .query_param(Parameter()
                         .key('fields')
                         .value(options.get('fields', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Order.from_dictionary)
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def orders_patch(self,
                     options=dict()):
        """Does a PATCH request to /v2/checkout/orders/{id}.

        Updates an order with a `CREATED` or `APPROVED` status. You cannot
        update an order with the `COMPLETED` status.<br/><br/>To make an
        update, you must provide a `reference_id`. If you omit this value with
        an order that contains only one purchase unit, PayPal sets the value
        to `default` which enables you to use the path:
        <code>\"/purchase_units/@reference_id=='default'/{attribute-or-object}\
        "</code>. Merchants and partners can add Level 2 and 3 data to
        payments to reduce risk and payment processing costs. For more
        information about processing payments, see <a
        href="https://developer.paypal.com/docs/checkout/advanced/processing/">
        checkout</a> or <a
        href="https://developer.paypal.com/docs/multiparty/checkout/advanced/pr
        ocessing/">multiparty checkout</a>.<blockquote><strong>Note:</strong>
        For error handling and troubleshooting, see <a
        href="https://developer.paypal.com/api/rest/reference/orders/v2/errors/
        #patch-order">Orders v2 errors</a>.</blockquote>Patchable attributes
        or
        objects:<br/><br/><table><thead><th>Attribute</th><th>Op</th><th>Notes<
        /th></thead><tbody><tr><td><code>intent</code></td><td>replace</td><td>
        </td></tr><tr><td><code>payer</code></td><td>replace,
        add</td><td>Using replace op for <code>payer</code> will replace the
        whole <code>payer</code> object with the value sent in
        request.</td></tr><tr><td><code>purchase_units</code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].custom_id</code></
        td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].description</co
        de></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].payee.email</co
        de></td><td>replace</td><td></td></tr><tr><td><code>purchase_units[].sh
        ipping.name</code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].shipping.email_add
        ress</code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].shipping.phone_num
        ber</code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].shipping.options</
        code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].shipping.address</
        code></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].shipping.type</cod
        e></td><td>replace,
        add</td><td></td></tr><tr><td><code>purchase_units[].soft_descriptor</c
        ode></td><td>replace,
        remove</td><td></td></tr><tr><td><code>purchase_units[].amount</code></
        td><td>replace</td><td></td></tr><tr><td><code>purchase_units[].items</
        code></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].invoice_id</cod
        e></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].payment_instruc
        tion</code></td><td>replace</td><td></td></tr><tr><td><code>purchase_un
        its[].payment_instruction.disbursement_mode</code></td><td>replace</td>
        <td>By default, <code>disbursement_mode</code> is
        <code>INSTANT</code>.</td></tr><tr><td><code>purchase_units[].payment_i
        nstruction.payee_receivable_fx_rate_id</code></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].payment_instruc
        tion.platform_fees</code></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].supplementary_d
        ata.airline</code></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>purchase_units[].supplementary_d
        ata.card</code></td><td>replace, add,
        remove</td><td></td></tr><tr><td><code>application_context.client_confi
        guration</code></td><td>replace,
        add</td><td></td></tr></tbody></table>

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order to update.
                    body -- List[Patch] -- TODO: type description here.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with an empty object in the JSON response body.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}')
            .http_method(HttpMethodEnum.PATCH)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def orders_confirm(self,
                       options=dict()):
        """Does a POST request to /v2/checkout/orders/{id}/confirm-payment-source.

        Payer confirms their intent to pay for the the Order with the given
        payment source.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order for which the payer
                        confirms their intent to pay.
                    pay_pal_client_metadata_id -- str -- TODO: type
                        description here.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value
                        is:<ul><li><code>return=minimal</code>. The server
                        returns a minimal response to optimize communication
                        between the API caller and the server. A minimal
                        response includes the <code>id</code>,
                        <code>status</code> and HATEOAS
                        links.</li><li><code>return=representation</code>. The
                        server returns a complete resource representation,
                        including the current state of the
                        resource.</li></ul>
                    body -- ConfirmOrderRequest -- TODO: type description
                        here.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request indicates that the payment source was added
                to the Order. A successful request returns the HTTP `200 OK`
                status code with a JSON response body that shows order
                details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}/confirm-payment-source')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Client-Metadata-Id')
                          .value(options.get('pay_pal_client_metadata_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Order.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def orders_authorize(self,
                         options=dict()):
        """Does a POST request to /v2/checkout/orders/{id}/authorize.

        Authorizes payment for an order. To successfully authorize payment for
        an order, the buyer must first approve the order or a valid
        payment_source must be provided in the request. A buyer can approve
        the order upon being redirected to the rel:approve URL that was
        returned in the HATEOAS links in the create order
        response.<blockquote><strong>Note:</strong> For error handling and
        troubleshooting, see <a
        href="https://developer.paypal.com/api/rest/reference/orders/v2/errors/
        #authorize-order">Orders v2 errors</a>.</blockquote>

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order for which to authorize.
                    pay_pal_request_id -- str -- The server stores keys for 6
                        hours. The API callers can request the times to up to
                        72 hours by speaking to their Account Manager.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value
                        is:<ul><li><code>return=minimal</code>. The server
                        returns a minimal response to optimize communication
                        between the API caller and the server. A minimal
                        response includes the <code>id</code>,
                        <code>status</code> and HATEOAS
                        links.</li><li><code>return=representation</code>. The
                        server returns a complete resource representation,
                        including the current state of the
                        resource.</li></ul>
                    pay_pal_client_metadata_id -- str -- TODO: type
                        description here.
                    pay_pal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see <a
                        href="https://developer.paypal.com/api/rest/requests/#p
                        aypal-auth-assertion">PayPal-Auth-Assertion</a>.
                    body -- OrderAuthorizeRequest -- TODO: type description
                        here.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful response to an idempotent request returns the HTTP
                `200 OK` status code with a JSON response body that shows
                authorized payment details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}/authorize')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('pay_pal_request_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('PayPal-Client-Metadata-Id')
                          .value(options.get('pay_pal_client_metadata_id', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('pay_pal_auth_assertion', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(OrderAuthorizeResponse.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The authorized payment failed due to insufficient permissions.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def orders_capture(self,
                       options=dict()):
        """Does a POST request to /v2/checkout/orders/{id}/capture.

        Captures payment for an order. To successfully capture payment for an
        order, the buyer must first approve the order or a valid
        payment_source must be provided in the request. A buyer can approve
        the order upon being redirected to the rel:approve URL that was
        returned in the HATEOAS links in the create order
        response.<blockquote><strong>Note:</strong> For error handling and
        troubleshooting, see <a
        href="https://developer.paypal.com/api/rest/reference/orders/v2/errors/
        #capture-order">Orders v2 errors</a>.</blockquote>

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order for which to capture a
                        payment.
                    pay_pal_request_id -- str -- The server stores keys for 6
                        hours. The API callers can request the times to up to
                        72 hours by speaking to their Account Manager.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value
                        is:<ul><li><code>return=minimal</code>. The server
                        returns a minimal response to optimize communication
                        between the API caller and the server. A minimal
                        response includes the <code>id</code>,
                        <code>status</code> and HATEOAS
                        links.</li><li><code>return=representation</code>. The
                        server returns a complete resource representation,
                        including the current state of the
                        resource.</li></ul>
                    pay_pal_client_metadata_id -- str -- TODO: type
                        description here.
                    pay_pal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see <a
                        href="https://developer.paypal.com/api/rest/requests/#p
                        aypal-auth-assertion">PayPal-Auth-Assertion</a>.
                    body -- OrderCaptureRequest -- TODO: type description
                        here.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful response to an idempotent request returns the HTTP
                `200 OK` status code with a JSON response body that shows
                captured payment details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}/capture')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('pay_pal_request_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('PayPal-Client-Metadata-Id')
                          .value(options.get('pay_pal_client_metadata_id', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('pay_pal_auth_assertion', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Order.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The authorized payment failed due to insufficient permissions.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def orders_track_create(self,
                            options=dict()):
        """Does a POST request to /v2/checkout/orders/{id}/track.

        Adds tracking information for an Order.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order that the tracking
                        information is associated with.
                    body -- OrderTrackerRequest -- TODO: type description
                        here.
                    pay_pal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see <a
                        href="https://developer.paypal.com/api/rest/requests/#p
                        aypal-auth-assertion">PayPal-Auth-Assertion</a>.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful response to an idempotent request returns the HTTP
                `200 OK` status code with a JSON response body that shows
                tracker details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}/track')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('pay_pal_auth_assertion', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Order.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def orders_trackers_patch(self,
                              options=dict()):
        """Does a PATCH request to /v2/checkout/orders/{id}/trackers/{tracker_id}.

        Updates or cancels the tracking information for a PayPal order, by ID.
        Updatable attributes or
        objects:<br/><br/><table><thead><th>Attribute</th><th>Op</th><th>Notes<
        /th></thead><tbody></tr><tr><td><code>items</code></td><td>replace</td>
        <td>Using replace op for <code>items</code> will replace the entire
        <code>items</code> object with the value sent in
        request.</td></tr><tr><td><code>notify_payer</code></td><td>replace,
        add</td><td></td></tr><tr><td><code>status</code></td><td>replace</td><
        td>Only patching status to CANCELLED is currently
        supported.</td></tr></tbody></table>

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    id -- str -- The ID of the order that the tracking
                        information is associated with.
                    tracker_id -- str -- The order tracking ID.
                    body -- List[Patch] -- TODO: type description here.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP `204 No Content` status
                code with an empty object in the JSON response body.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/checkout/orders/{id}/trackers/{tracker_id}')
            .http_method(HttpMethodEnum.PATCH)
            .template_param(Parameter()
                            .key('id')
                            .value(options.get('id', None))
                            .should_encode(True))
            .template_param(Parameter()
                            .key('tracker_id')
                            .value(options.get('tracker_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.authorization_with_additional_data import AuthorizationWithAdditionalData
from paypalserversdk.models.capture import Capture
from paypalserversdk.models.refund import Refund


class PaymentCollection(object):

    """Implementation of the 'Payment Collection' model.

    The collection of payments, or transactions, for a purchase unit in an
    order. For example, authorized payments, captured payments, and refunds.

    Attributes:
        authorizations (List[AuthorizationWithAdditionalData]): An array of
            authorized payments for a purchase unit. A purchase unit can have
            zero or more authorized payments.
        captures (List[Capture]): An array of captured payments for a purchase
            unit. A purchase unit can have zero or more captured payments.
        refunds (List[Refund]): An array of refunds for a purchase unit. A
            purchase unit can have zero or more refunds.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "authorizations": 'authorizations',
        "captures": 'captures',
        "refunds": 'refunds'
    }

    _optionals = [
        'authorizations',
        'captures',
        'refunds',
    ]

    def __init__(self,
                 authorizations=APIHelper.SKIP,
                 captures=APIHelper.SKIP,
                 refunds=APIHelper.SKIP):
        """Constructor for the PaymentCollection class"""

        # Initialize members of the class
        if authorizations is not APIHelper.SKIP:
            self.authorizations = authorizations 
        if captures is not APIHelper.SKIP:
            self.captures = captures 
        if refunds is not APIHelper.SKIP:
            self.refunds = refunds 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        authorizations = None
        if dictionary.get('authorizations') is not None:
            authorizations = [AuthorizationWithAdditionalData.from_dictionary(x) for x in dictionary.get('authorizations')]
        else:
            authorizations = APIHelper.SKIP
        captures = None
        if dictionary.get('captures') is not None:
            captures = [Capture.from_dictionary(x) for x in dictionary.get('captures')]
        else:
            captures = APIHelper.SKIP
        refunds = None
        if dictionary.get('refunds') is not None:
            refunds = [Refund.from_dictionary(x) for x in dictionary.get('refunds')]
        else:
            refunds = APIHelper.SKIP
        # Return an object of this model
        return cls(authorizations,
                   captures,
                   refunds)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class P24PaymentObject(object):

    """Implementation of the 'P24 Payment Object' model.

    Information used to pay using P24(Przelewy24).

    Attributes:
        name (str): The full name representation like Mr J Smith.
        email (str): The internationalized email
            address.<blockquote><strong>Note:</strong> Up to 64 characters are
            allowed before and 255 characters are allowed after the
            <code>@</code> sign. However, the generally accepted maximum
            length for an email address is 254 characters. The pattern
            verifies that an unquoted <code>@</code> sign
            exists.</blockquote>
        country_code (str): The [two-character ISO 3166-1
            code](/api/rest/reference/country-codes/) that identifies the
            country or region.<blockquote><strong>Note:</strong> The country
            code for Great Britain is <code>GB</code> and not <code>UK</code>
            as used in the top-level domain names for that country. Use the
            `C2` country code for China worldwide for comparable uncontrolled
            price (CUP) method, bank card, and cross-border
            transactions.</blockquote>
        payment_descriptor (str): P24 generated payment description.
        method_id (str): Numeric identifier of the payment scheme or bank used
            for the payment.
        method_description (str): Friendly name of the payment scheme or bank
            used for the payment.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "email": 'email',
        "country_code": 'country_code',
        "payment_descriptor": 'payment_descriptor',
        "method_id": 'method_id',
        "method_description": 'method_description'
    }

    _optionals = [
        'name',
        'email',
        'country_code',
        'payment_descriptor',
        'method_id',
        'method_description',
    ]

    def __init__(self,
                 name=APIHelper.SKIP,
                 email=APIHelper.SKIP,
                 country_code=APIHelper.SKIP,
                 payment_descriptor=APIHelper.SKIP,
                 method_id=APIHelper.SKIP,
                 method_description=APIHelper.SKIP):
        """Constructor for the P24PaymentObject class"""

        # Initialize members of the class
        if name is not APIHelper.SKIP:
            self.name = name 
        if email is not APIHelper.SKIP:
            self.email = email 
        if country_code is not APIHelper.SKIP:
            self.country_code = country_code 
        if payment_descriptor is not APIHelper.SKIP:
            self.payment_descriptor = payment_descriptor 
        if method_id is not APIHelper.SKIP:
            self.method_id = method_id 
        if method_description is not APIHelper.SKIP:
            self.method_description = method_description 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        email = dictionary.get("email") if dictionary.get("email") else APIHelper.SKIP
        country_code = dictionary.get("country_code") if dictionary.get("country_code") else APIHelper.SKIP
        payment_descriptor = dictionary.get("payment_descriptor") if dictionary.get("payment_descriptor") else APIHelper.SKIP
        method_id = dictionary.get("method_id") if dictionary.get("method_id") else APIHelper.SKIP
        method_description = dictionary.get("method_description") if dictionary.get("method_description") else APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   email,
                   country_code,
                   payment_descriptor,
                   method_id,
                   method_description)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class GooglePayPaymentMethod(object):

    """Implementation of the 'Google Pay Payment Method' enum.

    The type of the payment credential. Currently, only CARD is supported.

    Attributes:
        CARD: TODO: type description here.

    """
    CARD = 'CARD'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PAResStatus(object):

    """Implementation of the 'PARes Status' enum.

    Transactions status result identifier. The outcome of the issuer's
    authentication.

    Attributes:
        Y: TODO: type description here.
        N: TODO: type description here.
        U: TODO: type description here.
        A: TODO: type description here.
        C: TODO: type description here.
        R: TODO: type description here.
        D: TODO: type description here.
        I: TODO: type description here.

    """
    Y = 'Y'

    N = 'N'

    U = 'U'

    A = 'A'

    C = 'C'

    R = 'R'

    D = 'D'

    I = 'I'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CaptureStatus(object):

    """Implementation of the 'Capture Status' enum.

    The status of the captured payment.

    Attributes:
        COMPLETED: TODO: type description here.
        DECLINED: TODO: type description here.
        PARTIALLY_REFUNDED: TODO: type description here.
        PENDING: TODO: type description here.
        REFUNDED: TODO: type description here.
        FAILED: TODO: type description here.

    """
    COMPLETED = 'COMPLETED'

    DECLINED = 'DECLINED'

    PARTIALLY_REFUNDED = 'PARTIALLY_REFUNDED'

    PENDING = 'PENDING'

    REFUNDED = 'REFUNDED'

    FAILED = 'FAILED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class Money(object):

    """Implementation of the 'Money' model.

    The currency and amount for a financial transaction, such as a balance or
    payment due.

    Attributes:
        currency_code (str): The [three-character ISO-4217 currency
            code](/api/rest/reference/currency-codes/) that identifies the
            currency.
        value (str): The value, which might be:<ul><li>An integer for
            currencies like `JPY` that are not typically fractional.</li><li>A
            decimal fraction for currencies like `TND` that are subdivided
            into thousandths.</li></ul>For the required number of decimal
            places for a currency code, see [Currency
            Codes](/api/rest/reference/currency-codes/).

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "currency_code": 'currency_code',
        "value": 'value'
    }

    def __init__(self,
                 currency_code=None,
                 value=None):
        """Constructor for the Money class"""

        # Initialize members of the class
        self.currency_code = currency_code 
        self.value = value 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        currency_code = dictionary.get("currency_code") if dictionary.get("currency_code") else None
        value = dictionary.get("value") if dictionary.get("value") else None
        # Return an object of this model
        return cls(currency_code,
                   value)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.address import Address
from paypalserversdk.models.setup_token_card_experience_context import SetupTokenCardExperienceContext


class SetupTokenRequestCard(object):

    """Implementation of the 'Setup Token Request Card' model.

    A Resource representing a request to vault a Card.

    Attributes:
        name (str): The card holder's name as it appears on the card.
        number (str): The primary account number (PAN) for the payment card.
        expiry (str): The year and month, in ISO-8601 `YYYY-MM` date format.
            See [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6).
        security_code (str): The three- or four-digit security code of the
            card. Also known as the CVV, CVC, CVN, CVE, or CID. This parameter
            cannot be present in the request when
            `payment_initiator=MERCHANT`.
        brand (CardBrand): The card network or brand. Applies to credit,
            debit, gift, and payment cards.
        billing_address (Address): The portable international postal address.
            Maps to
            [AddressValidationMetadata](https://github.com/googlei18n/libaddres
            sinput/wiki/AddressValidationMetadata) and HTML 5.1 [Autofilling
            form controls: the autocomplete
            attribute](https://www.w3.org/TR/html51/sec-forms.html#autofilling-
            form-controls-the-autocomplete-attribute).
        verification_method (str): The verification method of the card.
        experience_context (SetupTokenCardExperienceContext): Customizes the
            Vault creation flow experience for your customers.
        network_transaction_reference (object): TODO: type description here.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "number": 'number',
        "expiry": 'expiry',
        "security_code": 'security_code',
        "brand": 'brand',
        "billing_address": 'billing_address',
        "verification_method": 'verification_method',
        "experience_context": 'experience_context',
        "network_transaction_reference": 'network_transaction_reference'
    }

    _optionals = [
        'name',
        'number',
        'expiry',
        'security_code',
        'brand',
        'billing_address',
        'verification_method',
        'experience_context',
        'network_transaction_reference',
    ]

    def __init__(self,
                 name=APIHelper.SKIP,
                 number=APIHelper.SKIP,
                 expiry=APIHelper.SKIP,
                 security_code=APIHelper.SKIP,
                 brand=APIHelper.SKIP,
                 billing_address=APIHelper.SKIP,
                 verification_method=APIHelper.SKIP,
                 experience_context=APIHelper.SKIP,
                 network_transaction_reference=APIHelper.SKIP):
        """Constructor for the SetupTokenRequestCard class"""

        # Initialize members of the class
        if name is not APIHelper.SKIP:
            self.name = name 
        if number is not APIHelper.SKIP:
            self.number = number 
        if expiry is not APIHelper.SKIP:
            self.expiry = expiry 
        if security_code is not APIHelper.SKIP:
            self.security_code = security_code 
        if brand is not APIHelper.SKIP:
            self.brand = brand 
        if billing_address is not APIHelper.SKIP:
            self.billing_address = billing_address 
        if verification_method is not APIHelper.SKIP:
            self.verification_method = verification_method 
        if experience_context is not APIHelper.SKIP:
            self.experience_context = experience_context 
        if network_transaction_reference is not APIHelper.SKIP:
            self.network_transaction_reference = network_transaction_reference 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        number = dictionary.get("number") if dictionary.get("number") else APIHelper.SKIP
        expiry = dictionary.get("expiry") if dictionary.get("expiry") else APIHelper.SKIP
        security_code = dictionary.get("security_code") if dictionary.get("security_code") else APIHelper.SKIP
        brand = dictionary.get("brand") if dictionary.get("brand") else APIHelper.SKIP
        billing_address = Address.from_dictionary(dictionary.get('billing_address')) if 'billing_address' in dictionary.keys() else APIHelper.SKIP
        verification_method = dictionary.get("verification_method") if dictionary.get("verification_method") else APIHelper.SKIP
        experience_context = SetupTokenCardExperienceContext.from_dictionary(dictionary.get('experience_context')) if 'experience_context' in dictionary.keys() else APIHelper.SKIP
        network_transaction_reference = dictionary.get("network_transaction_reference") if dictionary.get("network_transaction_reference") else APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   number,
                   expiry,
                   security_code,
                   brand,
                   billing_address,
                   verification_method,
                   experience_context,
                   network_transaction_reference)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class Patch(object):

    """Implementation of the 'Patch' model.

    The JSON patch object to apply partial updates to resources.

    Attributes:
        op (PatchOp): The operation.
        path (str): The <a href="https://tools.ietf.org/html/rfc6901">JSON
            Pointer</a> to the target document location at which to complete
            the operation.
        value (object): The value to apply. The <code>remove</code>,
            <code>copy</code>, and <code>move</code> operations do not require
            a value. Since <a
            href="https://www.rfc-editor.org/rfc/rfc69021">JSON Patch</a>
            allows any type for <code>value</code>, the <code>type</code>
            property is not specified.
        mfrom (str): The <a href="https://tools.ietf.org/html/rfc6901">JSON
            Pointer</a> to the target document location from which to move the
            value. Required for the <code>move</code> operation.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "op": 'op',
        "path": 'path',
        "value": 'value',
        "mfrom": 'from'
    }

    _optionals = [
        'path',
        'value',
        'mfrom',
    ]

    def __init__(self,
                 op=None,
                 path=APIHelper.SKIP,
                 value=APIHelper.SKIP,
                 mfrom=APIHelper.SKIP):
        """Constructor for the Patch class"""

        # Initialize members of the class
        self.op = op 
        if path is not APIHelper.SKIP:
            self.path = path 
        if value is not APIHelper.SKIP:
            self.value = value 
        if mfrom is not APIHelper.SKIP:
            self.mfrom = mfrom 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        op = dictionary.get("op") if dictionary.get("op") else None
        path = dictionary.get("path") if dictionary.get("path") else APIHelper.SKIP
        value = dictionary.get("value") if dictionary.get("value") else APIHelper.SKIP
        mfrom = dictionary.get("from") if dictionary.get("from") else APIHelper.SKIP
        # Return an object of this model
        return cls(op,
                   path,
                   value,
                   mfrom)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class OrderApplicationContextShippingPreference(object):

    """Implementation of the 'Order Application Context Shipping Preference' enum.

    DEPRECATED. DEPRECATED. The shipping preference:<ul><li>Displays the
    shipping address to the customer.</li><li>Enables the customer to choose
    an address on the PayPal site.</li><li>Restricts the customer from
    changing the address during the payment-approval process.</li></ul>.  The
    fields in `application_context` are now available in the
    `experience_context` object under the `payment_source` which supports them
    (eg. `payment_source.paypal.experience_context.shipping_preference`).
    Please specify this field in the `experience_context` object instead of
    the `application_context` object.

    Attributes:
        GET_FROM_FILE: TODO: type description here.
        NO_SHIPPING: TODO: type description here.
        SET_PROVIDED_ADDRESS: TODO: type description here.

    """
    GET_FROM_FILE = 'GET_FROM_FILE'

    NO_SHIPPING = 'NO_SHIPPING'

    SET_PROVIDED_ADDRESS = 'SET_PROVIDED_ADDRESS'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.address import Address
from paypalserversdk.models.card_attributes import CardAttributes
from paypalserversdk.models.card_experience_context import CardExperienceContext
from paypalserversdk.models.card_stored_credential import CardStoredCredential
from paypalserversdk.models.network_token import NetworkToken


class CardRequest(object):

    """Implementation of the 'Card Request' model.

    The payment card to use to fund a payment. Can be a credit or debit
    card.<blockquote><strong>Note:</strong> Passing card number, cvv and
    expiry directly via the API requires <a
    href="https://www.pcisecuritystandards.org/pci_security/completing_self_ass
    essment"> PCI SAQ D compliance</a>. <br>*PayPal offers a mechanism by
    which you do not have to take on the <strong>PCI SAQ D</strong> burden by
    using hosted fields - refer to <a
    href="https://developer.paypal.com/docs/checkout/advanced/integrate/">this
    Integration Guide</a>*.</blockquote>

    Attributes:
        name (str): The card holder's name as it appears on the card.
        number (str): The primary account number (PAN) for the payment card.
        expiry (str): The year and month, in ISO-8601 `YYYY-MM` date format.
            See [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6).
        security_code (str): The three- or four-digit security code of the
            card. Also known as the CVV, CVC, CVN, CVE, or CID. This parameter
            cannot be present in the request when
            `payment_initiator=MERCHANT`.
        billing_address (Address): The portable international postal address.
            Maps to
            [AddressValidationMetadata](https://github.com/googlei18n/libaddres
            sinput/wiki/AddressValidationMetadata) and HTML 5.1 [Autofilling
            form controls: the autocomplete
            attribute](https://www.w3.org/TR/html51/sec-forms.html#autofilling-
            form-controls-the-autocomplete-attribute).
        attributes (CardAttributes): Additional attributes associated with the
            use of this card.
        vault_id (str): The PayPal-generated ID for the vaulted payment
            source. This ID should be stored on the merchant's server so the
            saved payment source can be used for future transactions.
        single_use_token (str): The PayPal-generated, short-lived,
            one-time-use token, used to communicate payment information to
            PayPal for transaction processing.
        stored_credential (CardStoredCredential): Provides additional details
            to process a payment using a `card` that has been stored or is
            intended to be stored (also referred to as stored_credential or
            card-on-file).<br/>Parameter
            compatibility:<br/><ul><li>`payment_type=ONE_TIME` is compatible
            only with `payment_initiator=CUSTOMER`.</li><li>`usage=FIRST` is
            compatible only with
            `payment_initiator=CUSTOMER`.</li><li>`previous_transaction_referen
            ce` or `previous_network_transaction_reference` is compatible only
            with `payment_initiator=MERCHANT`.</li><li>Only one of the
            parameters - `previous_transaction_reference` and
            `previous_network_transaction_reference` - can be present in the
            request.</li></ul>
        network_token (NetworkToken): The Third Party Network token used to
            fund a payment.
        experience_context (CardExperienceContext): Customizes the payer
            experience during the 3DS Approval for payment.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "number": 'number',
        "expiry": 'expiry',
        "security_code": 'security_code',
        "billing_address": 'billing_address',
        "attributes": 'attributes',
        "vault_id": 'vault_id',
        "single_use_token": 'single_use_token',
        "stored_credential": 'stored_credential',
        "network_token": 'network_token',
        "experience_context": 'experience_context'
    }

    _optionals = [
        'name',
        'number',
        'expiry',
        'security_code',
        'billing_address',
        'attributes',
        'vault_id',
        'single_use_token',
        'stored_credential',
        'network_token',
        'experience_context',
    ]

    def __init__(self,
                 name=APIHelper.SKIP,
                 number=APIHelper.SKIP,
                 expiry=APIHelper.SKIP,
                 security_code=APIHelper.SKIP,
                 billing_address=APIHelper.SKIP,
                 attributes=APIHelper.SKIP,
                 vault_id=APIHelper.SKIP,
                 single_use_token=APIHelper.SKIP,
                 stored_credential=APIHelper.SKIP,
                 network_token=APIHelper.SKIP,
                 experience_context=APIHelper.SKIP):
        """Constructor for the CardRequest class"""

        # Initialize members of the class
        if name is not APIHelper.SKIP:
            self.name = name 
        if number is not APIHelper.SKIP:
            self.number = number 
        if expiry is not APIHelper.SKIP:
            self.expiry = expiry 
        if security_code is not APIHelper.SKIP:
            self.security_code = security_code 
        if billing_address is not APIHelper.SKIP:
            self.billing_address = billing_address 
        if attributes is not APIHelper.SKIP:
            self.attributes = attributes 
        if vault_id is not APIHelper.SKIP:
            self.vault_id = vault_id 
        if single_use_token is not APIHelper.SKIP:
            self.single_use_token = single_use_token 
        if stored_credential is not APIHelper.SKIP:
            self.stored_credential = stored_credential 
        if network_token is not APIHelper.SKIP:
            self.network_token = network_token 
        if experience_context is not APIHelper.SKIP:
            self.experience_context = experience_context 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name") if dictionary.get("name") else APIHelper.SKIP
        number = dictionary.get("number") if dictionary.get("number") else APIHelper.SKIP
        expiry = dictionary.get("expiry") if dictionary.get("expiry") else APIHelper.SKIP
        security_code = dictionary.get("security_code") if dictionary.get("security_code") else APIHelper.SKIP
        billing_address = Address.from_dictionary(dictionary.get('billing_address')) if 'billing_address' in dictionary.keys() else APIHelper.SKIP
        attributes = CardAttributes.from_dictionary(dictionary.get('attributes')) if 'attributes' in dictionary.keys() else APIHelper.SKIP
        vault_id = dictionary.get("vault_id") if dictionary.get("vault_id") else APIHelper.SKIP
        single_use_token = dictionary.get("single_use_token") if dictionary.get("single_use_token") else APIHelper.SKIP
        stored_credential = CardStoredCredential.from_dictionary(dictionary.get('stored_credential')) if 'stored_credential' in dictionary.keys() else APIHelper.SKIP
        network_token = NetworkToken.from_dictionary(dictionary.get('network_token')) if 'network_token' in dictionary.keys() else APIHelper.SKIP
        experience_context = CardExperienceContext.from_dictionary(dictionary.get('experience_context')) if 'experience_context' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   number,
                   expiry,
                   security_code,
                   billing_address,
                   attributes,
                   vault_id,
                   single_use_token,
                   stored_credential,
                   network_token,
                   experience_context)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class Address(object):

    """Implementation of the 'Address' model.

    The portable international postal address. Maps to
    [AddressValidationMetadata](https://github.com/googlei18n/libaddressinput/w
    iki/AddressValidationMetadata) and HTML 5.1 [Autofilling form controls:
    the autocomplete
    attribute](https://www.w3.org/TR/html51/sec-forms.html#autofilling-form-con
    trols-the-autocomplete-attribute).

    Attributes:
        address_line_1 (str): The first line of the address, such as number
            and street, for example, `173 Drury Lane`. Needed for data entry,
            and Compliance and Risk checks. This field needs to pass the full
            address.
        address_line_2 (str): The second line of the address, for example, a
            suite or apartment number.
        admin_area_2 (str): A city, town, or village. Smaller than
            `admin_area_level_1`.
        admin_area_1 (str): The highest-level sub-division in a country, which
            is usually a province, state, or ISO-3166-2 subdivision. This data
            is formatted for postal delivery, for example, `CA` and not
            `California`. Value, by country, is:<ul><li>UK. A
            county.</li><li>US. A state.</li><li>Canada. A
            province.</li><li>Japan. A prefecture.</li><li>Switzerland. A
            *kanton*.</li></ul>
        postal_code (str): The postal code, which is the ZIP code or
            equivalent. Typically required for countries with a postal code or
            an equivalent. See [postal
            code](https://en.wikipedia.org/wiki/Postal_code).
        country_code (str): The [2-character ISO 3166-1
            code](/api/rest/reference/country-codes/) that identifies the
            country or region.<blockquote><strong>Note:</strong> The country
            code for Great Britain is <code>GB</code> and not <code>UK</code>
            as used in the top-level domain names for that country. Use the
            `C2` country code for China worldwide for comparable uncontrolled
            price (CUP) method, bank card, and cross-border
            transactions.</blockquote>

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "country_code": 'country_code',
        "address_line_1": 'address_line_1',
        "address_line_2": 'address_line_2',
        "admin_area_2": 'admin_area_2',
        "admin_area_1": 'admin_area_1',
        "postal_code": 'postal_code'
    }

    _optionals = [
        'address_line_1',
        'address_line_2',
        'admin_area_2',
        'admin_area_1',
        'postal_code',
    ]

    def __init__(self,
                 country_code=None,
                 address_line_1=APIHelper.SKIP,
                 address_line_2=APIHelper.SKIP,
                 admin_area_2=APIHelper.SKIP,
                 admin_area_1=APIHelper.SKIP,
                 postal_code=APIHelper.SKIP):
        """Constructor for the Address class"""

        # Initialize members of the class
        if address_line_1 is not APIHelper.SKIP:
            self.address_line_1 = address_line_1 
        if address_line_2 is not APIHelper.SKIP:
            self.address_line_2 = address_line_2 
        if admin_area_2 is not APIHelper.SKIP:
            self.admin_area_2 = admin_area_2 
        if admin_area_1 is not APIHelper.SKIP:
            self.admin_area_1 = admin_area_1 
        if postal_code is not APIHelper.SKIP:
            self.postal_code = postal_code 
        self.country_code = country_code 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        country_code = dictionary.get("country_code") if dictionary.get("country_code") else None
        address_line_1 = dictionary.get("address_line_1") if dictionary.get("address_line_1") else APIHelper.SKIP
        address_line_2 = dictionary.get("address_line_2") if dictionary.get("address_line_2") else APIHelper.SKIP
        admin_area_2 = dictionary.get("admin_area_2") if dictionary.get("admin_area_2") else APIHelper.SKIP
        admin_area_1 = dictionary.get("admin_area_1") if dictionary.get("admin_area_1") else APIHelper.SKIP
        postal_code = dictionary.get("postal_code") if dictionary.get("postal_code") else APIHelper.SKIP
        # Return an object of this model
        return cls(country_code,
                   address_line_1,
                   address_line_2,
                   admin_area_2,
                   admin_area_1,
                   postal_code)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class AuthorizationStatus(object):

    """Implementation of the 'Authorization Status' enum.

    The status for the authorized payment.

    Attributes:
        CREATED: TODO: type description here.
        CAPTURED: TODO: type description here.
        DENIED: TODO: type description here.
        PARTIALLY_CAPTURED: TODO: type description here.
        VOIDED: TODO: type description here.
        PENDING: TODO: type description here.

    """
    CREATED = 'CREATED'

    CAPTURED = 'CAPTURED'

    DENIED = 'DENIED'

    PARTIALLY_CAPTURED = 'PARTIALLY_CAPTURED'

    VOIDED = 'VOIDED'

    PENDING = 'PENDING'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.link_description import LinkDescription


class ErrorDetails(object):

    """Implementation of the 'Error Details' model.

    The error details. Required for client-side `4XX` errors.

    Attributes:
        field (str): The field that caused the error. If this field is in the
            body, set this value to the field's JSON pointer value. Required
            for client-side errors.
        value (str): The value of the field that caused the error.
        location (str): The location of the field that caused the error. Value
            is `body`, `path`, or `query`.
        issue (str): The unique, fine-grained application-level error code.
        links (List[LinkDescription]): An array of request-related [HATEOAS
            links](/api/rest/responses/#hateoas-links) that are either
            relevant to the issue by providing additional information or
            offering potential resolutions.
        description (str): The human-readable description for an issue. The
            description can change over the lifetime of an API, so clients
            must not depend on this value.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "issue": 'issue',
        "field": 'field',
        "value": 'value',
        "location": 'location',
        "links": 'links',
        "description": 'description'
    }

    _optionals = [
        'field',
        'value',
        'location',
        'links',
        'description',
    ]

    def __init__(self,
                 issue=None,
                 field=APIHelper.SKIP,
                 value=APIHelper.SKIP,
                 location='body',
                 links=APIHelper.SKIP,
                 description=APIHelper.SKIP):
        """Constructor for the ErrorDetails class"""

        # Initialize members of the class
        if field is not APIHelper.SKIP:
            self.field = field 
        if value is not APIHelper.SKIP:
            self.value = value 
        self.location = location 
        self.issue = issue 
        if links is not APIHelper.SKIP:
            self.links = links 
        if description is not APIHelper.SKIP:
            self.description = description 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        issue = dictionary.get("issue") if dictionary.get("issue") else None
        field = dictionary.get("field") if dictionary.get("field") else APIHelper.SKIP
        value = dictionary.get("value") if dictionary.get("value") else APIHelper.SKIP
        location = dictionary.get("location") if dictionary.get("location") else 'body'
        links = None
        if dictionary.get('links') is not None:
            links = [LinkDescription.from_dictionary(x) for x in dictionary.get('links')]
        else:
            links = APIHelper.SKIP
        description = dictionary.get("description") if dictionary.get("description") else APIHelper.SKIP
        # Return an object of this model
        return cls(issue,
                   field,
                   value,
                   location,
                   links,
                   description)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class StoredPaymentSourceUsageType(object):

    """Implementation of the 'Stored Payment Source Usage Type' enum.

    Indicates if this is a `first` or `subsequent` payment using a stored
    payment source (also referred to as stored credential or card on file).

    Attributes:
        FIRST: TODO: type description here.
        SUBSEQUENT: TODO: type description here.
        DERIVED: TODO: type description here.

    """
    FIRST = 'FIRST'

    SUBSEQUENT = 'SUBSEQUENT'

    DERIVED = 'DERIVED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.exchange_rate import ExchangeRate
from paypalserversdk.models.money import Money


class NetAmountBreakdownItem(object):

    """Implementation of the 'Net Amount Breakdown Item' model.

    The net amount. Returned when the currency of the refund is different from
    the currency of the PayPal account where the merchant holds their funds.

    Attributes:
        payable_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        converted_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        exchange_rate (ExchangeRate): The exchange rate that determines the
            amount to convert from one currency to another currency.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "payable_amount": 'payable_amount',
        "converted_amount": 'converted_amount',
        "exchange_rate": 'exchange_rate'
    }

    _optionals = [
        'payable_amount',
        'converted_amount',
        'exchange_rate',
    ]

    def __init__(self,
                 payable_amount=APIHelper.SKIP,
                 converted_amount=APIHelper.SKIP,
                 exchange_rate=APIHelper.SKIP):
        """Constructor for the NetAmountBreakdownItem class"""

        # Initialize members of the class
        if payable_amount is not APIHelper.SKIP:
            self.payable_amount = payable_amount 
        if converted_amount is not APIHelper.SKIP:
            self.converted_amount = converted_amount 
        if exchange_rate is not APIHelper.SKIP:
            self.exchange_rate = exchange_rate 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        payable_amount = Money.from_dictionary(dictionary.get('payable_amount')) if 'payable_amount' in dictionary.keys() else APIHelper.SKIP
        converted_amount = Money.from_dictionary(dictionary.get('converted_amount')) if 'converted_amount' in dictionary.keys() else APIHelper.SKIP
        exchange_rate = ExchangeRate.from_dictionary(dictionary.get('exchange_rate')) if 'exchange_rate' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(payable_amount,
                   converted_amount,
                   exchange_rate)
